import platform
import unittest

import pytest
from mock import Mock

from conans.client.generators.text import TXTGenerator
from conans.model.build_info import CppInfo
from conans.model.conan_file import ConanFile
from conans.model.env_info import EnvValues
from conans.model.ref import ConanFileReference
from conans.model.settings import Settings


class AbsPathsTestCase(unittest.TestCase):

    @pytest.mark.skipif(platform.system() == "Windows", reason="Uses unix-like paths")
    def test_abs_path_unix(self):
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        ref = ConanFileReference.loads("pkg/0.1")
        cpp_info = CppInfo(ref.name, "/rootdir")
        cpp_info.includedirs = ["/an/absolute/dir"]
        cpp_info.filter_empty = False
        conanfile.deps_cpp_info.add(ref.name, cpp_info)

        master_content = TXTGenerator(conanfile).content
        after_cpp_info, _, _, _ = TXTGenerator.loads(master_content, filter_empty=False)
        self.assertListEqual(after_cpp_info[ref.name].includedirs, ["../an/absolute/dir"])
        self.assertListEqual(after_cpp_info[ref.name].include_paths, ["/rootdir/../an/absolute/dir"])

    @pytest.mark.skipif(platform.system() != "Windows", reason="Uses windows-like paths")
    def test_absolute_directory(self):
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        ref = ConanFileReference.loads("pkg/0.1")
        cpp_info = CppInfo(ref.name, "C:/my/root/path")
        cpp_info.includedirs = ["D:/my/path/to/something"]
        cpp_info.filter_empty = False
        conanfile.deps_cpp_info.add(ref.name, cpp_info)

        master_content = TXTGenerator(conanfile).content
        after_cpp_info, _, _, _ = TXTGenerator.loads(master_content, filter_empty=False)
        self.assertListEqual(after_cpp_info[ref.name].includedirs, ["D:/my/path/to/something"])
        self.assertListEqual(after_cpp_info[ref.name].include_paths, ["D:/my/path/to/something"])



from jinja2 import Template

_function_cpp = r"""
{% if name != "main" %}
#include "{{name}}.h"
{% endif %}

#include <iostream>

{% for it in includes -%}
#include "{{it}}.h"
{% endfor %}

int {{name}}(){
    #ifdef NDEBUG
    std::cout << "{{ msg or name }}: Release!\n";
    #else
    std::cout << "{{ msg or name }}: Debug!\n";
    #endif

    // ARCHITECTURES
    #ifdef _M_X64
    std::cout << "  {{ msg or name }} _M_X64 defined\n";
    #endif

    #ifdef _M_IX86
    std::cout << "  {{ msg or name }} _M_IX86 defined\n";
    #endif

    #ifdef _M_ARM64
    std::cout << "   {{ msg or name }} _M_ARM64 defined\n";
    #endif

    #if __i386__
    std::cout << "  {{ msg or name }} __i386__ defined\n";
    #endif

    #if __x86_64__
    std::cout << "  {{ msg or name }} __x86_64__ defined\n";
    #endif
  
    #if __aarch64__
    std::cout << "  {{ msg or name }} __aarch64__ defined\n";
    #endif

    // Libstdc++
    #if defined _GLIBCXX_USE_CXX11_ABI
    std::cout << "  {{ msg or name }} _GLIBCXX_USE_CXX11_ABI "<< _GLIBCXX_USE_CXX11_ABI << "\n";
    #endif

    // COMPILER VERSIONS
    #if _MSC_VER
    std::cout << "  {{ msg or name }} _MSC_VER" << _MSC_VER<< "\n";
    #endif

    #if _MSVC_LANG
    std::cout << "  {{ msg or name }} _MSVC_LANG" << _MSVC_LANG<< "\n";
    #endif

    #if __cplusplus
    std::cout << "  {{ msg or name }} __cplusplus" << __cplusplus<< "\n";
    #endif

    #if __INTEL_COMPILER
    std::cout << "  {{ msg or name }} __INTEL_COMPILER" << __INTEL_COMPILER<< "\n";
    #endif

    #if __GNUC__
    std::cout << "  {{ msg or name }} __GNUC__" << __GNUC__<< "\n";
    #endif

    #if __GNUC_MINOR__
    std::cout << "  {{ msg or name }} __GNUC_MINOR__" << __GNUC_MINOR__<< "\n";
    #endif

    #if __clang_major__
    std::cout << "  {{ msg or name }} __clang_major__" << __clang_major__<< "\n";
    #endif

    #if __clang_minor__
    std::cout << "  {{ msg or name }} __clang_minor__" << __clang_minor__<< "\n";
    #endif

    #if __apple_build_version__
    std::cout << "  {{ msg or name }} __apple_build_version__" << __apple_build_version__<< "\n";
    #endif

    // SUBSYSTEMS

    #if __MSYS__
    std::cout << "  {{ msg or name }} __MSYS__" << __MSYS__<< "\n";
    #endif

    #if __MINGW32__
    std::cout << "  {{ msg or name }} __MINGW32__" << __MINGW32__<< "\n";
    #endif

    #if __MINGW64__
    std::cout << "  {{ msg or name }} __MINGW64__" << __MINGW64__<< "\n";
    #endif

    #if __CYGWIN__
    std::cout << "  {{ msg or name }} __CYGWIN__" << __CYGWIN__<< "\n";
    #endif

    {% for it in preprocessor -%}
    std::cout << "  {{msg}} {{it}}: " <<  {{it}} << "\n";
    {%- endfor %}

    {% for it in calls -%}
    {{it}}();
    {% endfor %}
    return 0;
}
"""


def gen_function_cpp(**context):
    t = Template(_function_cpp)
    return t.render(**context)


_function_h = """
#pragma once

{% for it in includes -%}
#include "{{it}}.h"
{%- endfor %}

#ifdef _WIN32
  #define {{name.upper()}}_EXPORT __declspec(dllexport)
#else
  #define {{name.upper()}}_EXPORT
#endif
{{name.upper()}}_EXPORT int {{name}}();
"""


def gen_function_h(**context):
    t = Template(_function_h)
    return t.render(**context)



import unittest

from parameterized import parameterized

from conans.test.utils.tools import TestClient, GenConanfile


class BuildRequiresTestCase(unittest.TestCase):

    @parameterized.expand([(True,), (False,)])
    def test_build_require_lib(self, use_single_profile):
        t = TestClient()
        t.save({'br_lib.py': GenConanfile("br_lib", "v1").with_provides("libjpeg"),
                'br.py': GenConanfile("br", "v1").with_require("br_lib/v1"),
                'app.py': GenConanfile("app", "v1").with_build_requires("br/v1")
                                                   .with_provides("libjpeg")})
        t.run("create br_lib.py")
        t.run("create br.py")
        if use_single_profile:
            t.run("install app.py", assert_error=True)
            self.assertIn(" - 'libjpeg' provided by 'app.py (app/v1)', 'br_lib/v1'", t.out)
        else:
            t.run("install app.py --profile:host=default --profile:build=default")

    @parameterized.expand([(True,), (False,)])
    def test_build_require_host(self, use_single_profile):
        t = TestClient()
        t.save({'br_lib.py': GenConanfile("br_lib", "v1").with_provides("libjpeg"),
                'br.py': GenConanfile("br", "v1").with_require("br_lib/v1"),
                'app.py': GenConanfile("app", "v1").with_build_requirement("br/v1",
                                                                           force_host_context=True)
                                                   .with_provides("libjpeg")})
        t.run("create br_lib.py")
        t.run("create br.py")
        if use_single_profile:
            t.run("install app.py", assert_error=True)
        else:
            t.run("install app.py --profile:host=default --profile:build=default", assert_error=True)
        self.assertIn(" - 'libjpeg' provided by 'app.py (app/v1)', 'br_lib/v1'", t.out)

    @parameterized.expand([(True,), (False,)])
    def test_build_require_host_transitive(self, use_single_profile):
        t = TestClient()
        t.save({'br.py': GenConanfile("br", "v1").with_provides("libjpeg"),
                'lib.py': GenConanfile("lib", "v1").with_build_requirement("br/v1",
                                                                           force_host_context=True),
                'app.py': GenConanfile("app", "v1").with_require("lib/v1")
                                                   .with_provides("libjpeg")})
        t.run("export br.py")
        t.run("export lib.py")
        if use_single_profile:
            t.run("install app.py --build")
        else:
            t.run("install app.py --profile:host=default --profile:build=default --build")

    @parameterized.expand([(True,), (False,)])
    def test_build_require_branches(self, use_single_profile):
        t = TestClient()
        t.save({'br_lhs.py': GenConanfile("br_lhs", "v1").with_provides("libjpeg"),
                'br_rhs.py': GenConanfile("br_rhs", "v1").with_provides("libjpeg"),
                'app.py': GenConanfile("app", "v1").with_build_requires("br_lhs/v1")
                                                   .with_build_requires("br_rhs/v1")})
        t.run("create br_lhs.py")
        t.run("create br_rhs.py")
        if use_single_profile:
            t.run("install app.py", assert_error=True)
            self.assertIn(" - 'libjpeg' provided by 'br_lhs/v1', 'br_rhs/v1'", t.out)
        else:
            t.run("install app.py --profile:host=default --profile:build=default")

    def test_build_require_of_build_require(self):
        # Only makes sense for two profiles
        t = TestClient()
        t.save({'br_nested.py': GenConanfile("br_nested", "v1").with_provides("libjpeg"),
                'br.py': GenConanfile("br", "v1").with_provides("libjpeg")
                                                 .with_build_requires("br_nested/v1"),
                'app.py': GenConanfile("app", "v1").with_provides("libjpeg")
                                                   .with_build_requires("br/v1")})
        t.run("export br_nested.py")
        t.run("export br.py")
        t.run("install app.py --profile:host=default --profile:build=default --build")



import textwrap

from conans.test.utils.conan_v2_tests import ConanV2ModeTestCase


class DefaultOptionsSyntaxTestCase(ConanV2ModeTestCase):
    msg = "Conan v2 incompatible: Declare 'default_options' as a dictionary"

    def test_deprecate_string(self):
        t = self.get_client()
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class Recipe(ConanFile):
                options = {"option": [True, False]}
                default_options = "option=True"
        """)
        t.save({'conanfile.py': conanfile})
        t.run('create . name/version@', assert_error=True)
        self.assertIn(self.msg, t.out)

        t.run('inspect . -a default_options', assert_error=True)
        self.assertIn(self.msg, t.out)

    def test_deprecate_list(self):
        t = self.get_client()
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class Recipe(ConanFile):
                options = {"option": [True, False], "option2": [False, ]}
                default_options = "option=True", "option2=False"
        """)
        t.save({'conanfile.py': conanfile})
        t.run('create . name/version@', assert_error=True)
        self.assertIn(self.msg, t.out)

        t.run('inspect . -a default_options', assert_error=True)
        self.assertIn(self.msg, t.out)

    def test_deprecate_tuple(self):
        t = self.get_client()
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class Recipe(ConanFile):
                options = {"option": [True, False], "option2": [False, ]}
                default_options = ("option=True", "option2=False", )
        """)
        t.save({'conanfile.py': conanfile})
        t.run('create . name/version@', assert_error=True)
        self.assertIn(self.msg, t.out)

        t.run('inspect . -a default_options', assert_error=True)
        self.assertIn(self.msg, t.out)



import textwrap
import unittest

import pytest

from conans.model.ref import ConanFileReference
from conans.test.utils.tools import TestClient


class ExportMetadataTest(unittest.TestCase):
    conanfile = textwrap.dedent("""
        from conans import ConanFile

        class Lib(ConanFile):
            revision_mode = "{revision_mode}"
    """)

    summary_hash = "bfe8b4a6a2a74966c0c4e0b34705004a"

    @pytest.mark.tool_git
    def test_revision_mode_scm(self):
        t = TestClient()
        commit = t.init_git_repo({'conanfile.py': self.conanfile.format(revision_mode="scm")})

        ref = ConanFileReference.loads("name/version@user/channel")
        t.run("export . {}".format(ref))

        meta = t.cache.package_layout(ref, short_paths=False).load_metadata()
        self.assertEqual(meta.recipe.revision, commit)



import os
import platform
import textwrap
import unittest

import pytest

from conans.client.tools import replace_in_file
from conans.model.ref import ConanFileReference
from conans.test.utils.tools import TestClient, GenConanfile, TurboTestClient


@pytest.mark.tool_cmake
class CMakeGeneratorTest(unittest.TestCase):

    def test_no_check_compiler(self):
        # https://github.com/conan-io/conan/issues/4268
        file_content = textwrap.dedent("""
            from conans import ConanFile, CMake

            class ConanFileToolsTest(ConanFile):
                generators = "cmake"

                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                """)

        cmakelists = textwrap.dedent("""
            cmake_minimum_required(VERSION 2.8)
            PROJECT(conanzlib LANGUAGES NONE)
            set(CONAN_DISABLE_CHECK_COMPILER TRUE)

            include(conanbuildinfo.cmake)
            CONAN_BASIC_SETUP()
            """)
        client = TestClient()
        client.save({"conanfile.py": file_content,
                     "CMakeLists.txt": cmakelists})

        client.run('install .')
        client.run('build .')
        self.assertIn("WARN: Disabled conan compiler checks", client.out)

    @pytest.mark.skipif(platform.system() != "Linux", reason="Only linux")
    def test_check_compiler_package_id(self):
        # https://github.com/conan-io/conan/issues/6658
        file_content = textwrap.dedent("""
            from conans import ConanFile, CMake

            class ConanFileToolsTest(ConanFile):
                settings = "os", "compiler", "arch", "build_type"
                generators = "cmake"
                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                def package_id(self):
                    self.info.settings.compiler.version = "SomeVersion"
                """)

        cmakelists = textwrap.dedent("""
            cmake_minimum_required(VERSION 2.8)
            project(conanzlib)
            include(conanbuildinfo.cmake)
            conan_basic_setup()
            """)
        client = TestClient()
        client.save({"conanfile.py": file_content,
                     "CMakeLists.txt": cmakelists})

        client.run('install .')
        client.run_command('cmake .')
        self.assertIn("Conan: Checking correct version:", client.out)

    @pytest.mark.slow
    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
    def test_skip_check_if_toolset(self):
        file_content = textwrap.dedent("""
            from conans import ConanFile, CMake

            class ConanFileToolsTest(ConanFile):
                generators = "cmake"
                exports_sources = "CMakeLists.txt"
                settings = "os", "arch", "compiler"

                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                    cmake.build()
                """)
        client = TestClient()
        cmakelists = textwrap.dedent("""
            set(CMAKE_CXX_COMPILER_WORKS 1)
            set(CMAKE_CXX_ABI_COMPILED 1)
            PROJECT(Hello)
            cmake_minimum_required(VERSION 2.8)
            include("${CMAKE_BINARY_DIR}/conanbuildinfo.cmake")
            CONAN_BASIC_SETUP()
            """)
        client.save({"conanfile.py": file_content,
                     "CMakeLists.txt": cmakelists})
        client.run("create . lib/1.0@ -s compiler='Visual Studio' -s compiler.toolset=v140")
        self.assertIn("Conan: Skipping compiler check: Declared 'compiler.toolset'", client.out)

    @pytest.mark.slow
    @pytest.mark.tool_visual_studio(version="17")
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires Visual Studio")
    def test_check_msvc_compiler(self):
        """
        Checking if MSVC 19.X compiler is being called via CMake
        while using compiler=msvc in Conan profile.

        Issue related: https://github.com/conan-io/conan/issues/10185
        """
        file_content = textwrap.dedent("""
            from conans import ConanFile, CMake

            class ConanFileMSVCTest(ConanFile):
                generators = "cmake"
                exports_sources = "CMakeLists.txt"
                settings = "os", "arch", "compiler"

                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                    cmake.build()
                """)
        client = TestClient()
        cmakelists = textwrap.dedent("""
            PROJECT(Hello)
            cmake_minimum_required(VERSION 2.8)
            include("${CMAKE_BINARY_DIR}/conanbuildinfo.cmake")
            CONAN_BASIC_SETUP()
            """)
        client.save({"conanfile.py": file_content,
                     "CMakeLists.txt": cmakelists})
        client.run("create . lib/1.0@ -s compiler=msvc -s compiler.version=193")
        self.assertIn("-- The C compiler identification is MSVC 19.3", client.out)
        self.assertIn("-- The CXX compiler identification is MSVC 19.3", client.out)

    @pytest.mark.slow
    def test_no_output(self):
        client = TestClient()
        client.run("new Test/1.0 --sources")
        cmakelists_path = os.path.join(client.current_folder, "src", "CMakeLists.txt")

        # Test output works as expected
        client.run("install .")
        # No need to do a full create, the build --configure is good
        client.run("build . --configure")
        self.assertIn("Conan: Using cmake global configuration", client.out)
        self.assertIn("Conan: Adjusting default RPATHs Conan policies", client.out)
        self.assertIn("Conan: Adjusting language standard", client.out)

        # Silence output
        replace_in_file(cmakelists_path,
                        "conan_basic_setup()",
                        "set(CONAN_CMAKE_SILENT_OUTPUT True)\nconan_basic_setup()",
                        output=client.out)
        client.run("build . --configure")
        self.assertNotIn("Conan: Using cmake global configuration", client.out)
        self.assertNotIn("Conan: Adjusting default RPATHs Conan policies", client.out)
        self.assertNotIn("Conan: Adjusting language standard", client.out)

        # Use TARGETS
        replace_in_file(cmakelists_path, "conan_basic_setup()", "conan_basic_setup(TARGETS)",
                        output=client.out)
        client.run("build . --configure")
        self.assertNotIn("Conan: Using cmake targets configuration", client.out)
        self.assertNotIn("Conan: Adjusting default RPATHs Conan policies", client.out)
        self.assertNotIn("Conan: Adjusting language standard", client.out)

    def test_system_libs(self):
        mylib = textwrap.dedent("""
            import os
            from conans import ConanFile

            class MyLib(ConanFile):
                settings = "os", "compiler", "arch", "build_type"
                exports_sources = "*"

                def package(self):
                    self.copy("*", dst="lib")

                def package_info(self):
                    self.cpp_info.system_libs = ["sys1"]
                    self.cpp_info.libs = ["lib1"]
                """)
        consumer = textwrap.dedent("""
            import os
            from conans import ConanFile

            class Consumer(ConanFile):
                requires = "mylib/1.0@us/ch"
                generators = "cmake"
                """)
        cmakelists = textwrap.dedent("""
            cmake_minimum_required(VERSION 3.1)
            project(consumer CXX)
            include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
            conan_basic_setup(TARGETS)
            get_target_property(tmp CONAN_PKG::mylib INTERFACE_LINK_LIBRARIES)
            message("Target libs: ${tmp}")
            get_target_property(tmpp CONAN_LIB::mylib_lib1 INTERFACE_LINK_LIBRARIES)
            message("CONAN_LIB::mylib_lib1 system libs: ${tmpp}")
            """)
        client = TestClient()
        client.save({"conanfile_mylib.py": mylib, "conanfile_consumer.py": consumer,
                     "CMakeLists.txt": cmakelists, "lib1.lib": "", "liblib1.a": ""})
        client.run("create conanfile_mylib.py mylib/1.0@us/ch")
        client.run("install conanfile_consumer.py")

        content = client.load("conanbuildinfo.cmake")
        self.assertIn("set(CONAN_LIBS ${CONAN_LIBS} ${CONAN_SYSTEM_LIBS} ${CONAN_FRAMEWORKS_FOUND})",
                      content)
        self.assertIn("set(CONAN_LIBS_MYLIB ${CONAN_PKG_LIBS_MYLIB} "
                      "${CONAN_SYSTEM_LIBS_MYLIB} ${CONAN_FRAMEWORKS_FOUND_MYLIB})",
                      content)
        self.assertIn("set(CONAN_PKG_LIBS_MYLIB lib1)", content)
        self.assertIn("set(CONAN_SYSTEM_LIBS sys1 ${CONAN_SYSTEM_LIBS})", content)
        self.assertIn("set(CONAN_SYSTEM_LIBS_MYLIB sys1)", content)

        # Check target has libraries and system deps available
        client.run_command("cmake .")
        self.assertIn("Target libs: CONAN_LIB::mylib_lib1;sys1;$", client.out)
        self.assertIn("CONAN_LIB::mylib_lib1 system libs: ;sys1;;", client.out)

    def test_targets_system_libs(self):
        mylib = GenConanfile().with_package_info(cpp_info={"libs": ["lib1", "lib11"],
                                                           "system_libs": ["sys1"]},
                                                 env_info={})\
            .with_package_file("lib/lib1.lib", " ").with_package_file("lib/liblib1.a", " ")\
            .with_package_file("lib/lib11.lib", " ").with_package_file("lib/liblib11.a", " ")
        mylib_ref = ConanFileReference("mylib", "1.0", "us", "ch")

        myotherlib = GenConanfile().with_package_info(cpp_info={"libs": ["lib2"],
                                                                "system_libs": ["sys2"]},
                                                      env_info={}).with_require(mylib_ref) \
            .with_package_file("lib/lib2.lib", " ").with_package_file("lib/liblib2.a", " ")
        myotherlib_ref = ConanFileReference("myotherlib", "1.0", "us", "ch")

        client = TurboTestClient()
        client.create(mylib_ref, mylib)
        client.create(myotherlib_ref, myotherlib)

        consumer = textwrap.dedent("""
            import os
            from conans import ConanFile, CMake

            class Consumer(ConanFile):
                requires = "myotherlib/1.0@us/ch"
                generators = "cmake"
                settings = "os", "compiler", "arch", "build_type"
                exports_sources = ["CMakeLists.txt"]

                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                    cmake.build()
                """)
        cmakelists = textwrap.dedent("""
            cmake_minimum_required(VERSION 3.1)
            project(consumer CXX)
            include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
            conan_basic_setup("TARGETS")

            get_target_property(ml_pkg_libs CONAN_PKG::mylib INTERFACE_LINK_LIBRARIES)
            message("CONAN_PKG::mylib libs: ${ml_pkg_libs}")
            get_target_property(ml_lib1_libs CONAN_LIB::mylib_lib1 INTERFACE_LINK_LIBRARIES)
            message("CONAN_LIB::mylib_lib1 libs: ${ml_lib1_libs}")
            get_target_property(ml_lib11_libs CONAN_LIB::mylib_lib11 INTERFACE_LINK_LIBRARIES)
            message("CONAN_LIB::mylib_lib11 libs: ${ml_lib11_libs}")

            get_target_property(mol_pkg_libs CONAN_PKG::myotherlib INTERFACE_LINK_LIBRARIES)
            message("CONAN_PKG::myotherlib libs: ${mol_pkg_libs}")
            get_target_property(ml_lib2_libs CONAN_LIB::myotherlib_lib2 INTERFACE_LINK_LIBRARIES)
            message("CONAN_LIB::myotherlib_lib2 libs: ${ml_lib2_libs}")
            """)

        client.save({"conanfile.py": consumer, "CMakeLists.txt": cmakelists})
        client.run("create conanfile.py consumer/1.0@us/ch")

        self.assertNotIn("Library sys1 not found in package, might be system one", client.out)
        self.assertIn("CONAN_PKG::mylib libs: "
                      "CONAN_LIB::mylib_lib1;CONAN_LIB::mylib_lib11;sys1;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:>;"
                      "$<$<CONFIG:Release>:;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:>>;"
                      "$<$<CONFIG:RelWithDebInfo>:;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:>>;"
                      "$<$<CONFIG:MinSizeRel>:;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:>>;"
                      "$<$<CONFIG:Debug>:;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:>>",
                      client.out)
        self.assertIn("CONAN_LIB::mylib_lib1 libs: ;sys1;;", client.out)
        self.assertIn("CONAN_LIB::mylib_lib11 libs: ;sys1;;", client.out)

        self.assertNotIn("Library sys2 not found in package, might be system one", client.out)
        self.assertIn("CONAN_PKG::myotherlib libs: "
                      "CONAN_LIB::myotherlib_lib2;sys2;CONAN_PKG::mylib;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:>;"
                      "$<$<CONFIG:Release>:;CONAN_PKG::mylib;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:>>;"
                      "$<$<CONFIG:RelWithDebInfo>:;CONAN_PKG::mylib;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:>>;"
                      "$<$<CONFIG:MinSizeRel>:;CONAN_PKG::mylib;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:>>;"
                      "$<$<CONFIG:Debug>:;CONAN_PKG::mylib;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:>;"
                      "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:>>",
                      client.out)
        self.assertIn("CONAN_LIB::myotherlib_lib2 libs: ;sys2;;CONAN_PKG::mylib", client.out)

    def test_user_appended_libs(self):
        conanfile = textwrap.dedent("""
            import os
            from conans import ConanFile, CMake

            class MyLib(ConanFile):
                name = "mylib"
                version = "1.0"
                settings = "os", "compiler", "arch", "build_type"
                exports_sources = "lib1.a", "lib1.lib"

                def package(self):
                    self.copy("*", dst="lib")

                def package_info(self):
                    self.cpp_info.system_libs = ["sys1"]
                    self.cpp_info.libs = ["lib1"]
                """)
        consumer = textwrap.dedent("""
            import os
            from conans import ConanFile, CMake

            class Consumer(ConanFile):
                name = "consumer"
                version = "1.0"
                generators = "cmake"
                settings = "os", "compiler", "arch", "build_type"
                exports_sources = "CMakeLists.txt"
                requires = "mylib/1.0"

                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                """)
        cmakelists = textwrap.dedent("""
            cmake_minimum_required(VERSION 3.1)
            project(consumer CXX)
            set(CONAN_LIBS additional_lib)
            set(CONAN_SYSTEM_LIBS additional_sys)
            include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
            message("CONAN_LIBS: ${CONAN_LIBS}")
            message("CONAN_SYSTEM_LIBS: ${CONAN_SYSTEM_LIBS}")
            """)
        client = TestClient()
        client.save({"conanfile.py": conanfile, "lib1.lib": "", "liblib1.a": "",
                     "consumer.py": consumer, "CMakeLists.txt": cmakelists})
        client.run("create .")
        client.run("create consumer.py")
        self.assertIn("CONAN_LIBS: lib1;additional_lib;sys1;additional_sys", client.out)
        self.assertIn("CONAN_SYSTEM_LIBS: sys1;additional_sys", client.out)

    def test_conan_get_policy(self):
        # https://github.com/conan-io/conan/issues/6974
        file_content = textwrap.dedent("""
            from conans import ConanFile, CMake

            class ConanFileToolsTest(ConanFile):
                name = "consumer"
                version = "1.0"
                generators = "cmake"
                settings = "os", "compiler", "arch", "build_type"
                exports_sources = "CMakeLists.txt"

                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                """)

        cmakelists = textwrap.dedent("""
            cmake_minimum_required(VERSION 2.8)
            PROJECT(conanzlib LANGUAGES NONE)
            set(CONAN_DISABLE_CHECK_COMPILER TRUE)
            # test with any old build policy as we may
            # not have cmake version that supports 091
            cmake_policy(SET CMP0054 OLD)
            include(conanbuildinfo.cmake)
            conan_get_policy(CMP0054 policy_0054)
            message("POLICY CMP0054 IS ${policy_0054}")
            CONAN_BASIC_SETUP()
            """)
        client = TestClient()
        client.save({"conanfile.py": file_content,
                     "CMakeLists.txt": cmakelists})

        client.run('create .')
        self.assertIn("POLICY CMP0054 IS OLD", client.out)

    def test_do_not_mix_cflags_cxxflags(self):
        client = TestClient()

        def run_test(consumer_generator, consumer_cmakelists, with_components=True):

            def generate_files(upstream_cpp_info, consumer_generator, consumer_cmakelists):
                upstream_conanfile = GenConanfile().with_name("upstream").with_version("1.0")\
                    .with_package_info(cpp_info=upstream_cpp_info, env_info={})
                client.save({"conanfile.py": upstream_conanfile}, clean_first=True)
                client.run("create .")
                consumer_conanfile = textwrap.dedent("""
                    from conans import ConanFile, CMake

                    class Consumer(ConanFile):
                        name = "consumer"
                        version = "1.0"
                        settings = "os", "compiler", "arch", "build_type"
                        exports_sources = "CMakeLists.txt"
                        requires = "upstream/1.0"
                        generators = "{}"

                        def build(self):
                            cmake = CMake(self)
                            cmake.configure()
                    """)
                client.save({"conanfile.py": consumer_conanfile.format(consumer_generator),
                             "CMakeLists.txt": consumer_cmakelists})
                client.run("create .")

            if consumer_generator in ["cmake_find_package", "cmake_find_package_multi"]:
                if with_components:
                    cpp_info = {"components": {"comp": {"cflags": ["one", "two"],
                                                        "cxxflags": ["three", "four"]}}}
                else:
                    cpp_info = {"cflags": ["one", "two"], "cxxflags": ["three", "four"]}
                generate_files(cpp_info, consumer_generator, consumer_cmakelists)
                self.assertIn("compile options: three;four;one;two", client.out)
                self.assertIn("cflags: one;two", client.out)
                self.assertIn("cxxflags: three;four", client.out)
                if with_components:
                    self.assertIn("comp cflags: one;two", client.out)
                    self.assertIn("comp cxxflags: three;four", client.out)
                    if consumer_generator == "cmake_find_package":
                        self.assertIn("comp compile options: one;two;three;four", client.out)
                    else:
                        self.assertIn("$<$<CONFIG:Debug>:;>;"
                                      "$<$<CONFIG:Release>:;one;two;three;four>;"
                                      "$<$<CONFIG:RelWithDebInfo>:;>;"
                                      "$<$<CONFIG:MinSizeRel>:;>"
                                      , client.out)
            else:
                generate_files({"cflags": ["one", "two"], "cxxflags": ["three", "four"]},
                               consumer_generator, consumer_cmakelists)
                self.assertIn("global cflags: one two", client.out)
                self.assertIn("global cxxflags: three four", client.out)
                self.assertIn("upstream cflags: one two", client.out)
                self.assertIn("upstream cxxflags: three four", client.out)

        # Test cmake generator
        cmakelists = textwrap.dedent("""
            cmake_minimum_required(VERSION 2.8)
            set(CMAKE_CXX_COMPILER_WORKS 1)
            project(consumer)
            include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
            message("global cflags: ${CONAN_C_FLAGS}")
            message("global cxxflags: ${CONAN_CXX_FLAGS}")
            message("upstream cflags: ${CONAN_C_FLAGS_UPSTREAM}")
            message("upstream cxxflags: ${CONAN_CXX_FLAGS_UPSTREAM}")
            """)
        run_test("cmake", cmakelists)

        # Test cmake_multi generator
        cmakelists = textwrap.dedent("""
            cmake_minimum_required(VERSION 2.8)
            set(CMAKE_CXX_COMPILER_WORKS 1)
            project(consumer)
            include(${CMAKE_BINARY_DIR}/conanbuildinfo_multi.cmake)
            message("global cflags: ${CONAN_C_FLAGS_RELEASE}")
            message("global cxxflags: ${CONAN_CXX_FLAGS_RELEASE}")
            message("upstream cflags: ${CONAN_C_FLAGS_UPSTREAM_RELEASE}")
            message("upstream cxxflags: ${CONAN_CXX_FLAGS_UPSTREAM_RELEASE}")
            """)
        run_test("cmake_multi", cmakelists)

        # Test cmake_find_package generator
        cmakelists = textwrap.dedent("""
            cmake_minimum_required(VERSION 2.8)
            set(CMAKE_CXX_COMPILER_WORKS 1)
            project(consumer)
            find_package(upstream)
            message("compile options: ${upstream_COMPILE_OPTIONS_LIST}")
            message("cflags: ${upstream_COMPILE_OPTIONS_C}")
            message("cxxflags: ${upstream_COMPILE_OPTIONS_CXX}")
            message("comp cflags: ${upstream_comp_COMPILE_OPTIONS_C}")
            message("comp cxxflags: ${upstream_comp_COMPILE_OPTIONS_CXX}")
            get_target_property(tmp upstream::comp INTERFACE_COMPILE_OPTIONS)
            message("comp compile options: ${tmp}")
            """)
        run_test("cmake_find_package", cmakelists)

        # Test cmake_find_package generator without components
        run_test("cmake_find_package", cmakelists, with_components=False)

        # Test cmake_find_package_multi generator
        cmakelists = textwrap.dedent("""
            cmake_minimum_required(VERSION 2.8)
            set(CMAKE_CXX_COMPILER_WORKS 1)
            project(consumer)
            find_package(upstream)
            message("compile options: ${upstream_COMPILE_OPTIONS_RELEASE_LIST}")
            message("cflags: ${upstream_COMPILE_OPTIONS_C_RELEASE}")
            message("cxxflags: ${upstream_COMPILE_OPTIONS_CXX_RELEASE}")
            message("comp cflags: ${upstream_comp_COMPILE_OPTIONS_C_RELEASE}")
            message("comp cxxflags: ${upstream_comp_COMPILE_OPTIONS_CXX_RELEASE}")
            get_target_property(tmp upstream::comp INTERFACE_COMPILE_OPTIONS)
            message("comp compile options: ${tmp}")
            """)
        run_test("cmake_find_package_multi", cmakelists)

        # Test cmake_find_package_multi generator without components
        run_test("cmake_find_package_multi", cmakelists, with_components=False)

    def test_build_modules_alias_target(self, use_components=False):
        client = TestClient()
        conanfile = textwrap.dedent("""
            import os
            from conans import ConanFile, CMake

            class Conan(ConanFile):
                name = "hello"
                version = "1.0"
                settings = "os", "arch", "compiler", "build_type"
                exports_sources = ["target-alias.cmake"]

                def package(self):
                    self.copy("target-alias.cmake", dst="share/cmake")

                def package_info(self):
                    module = os.path.join("share", "cmake", "target-alias.cmake")
                    self.cpp_info.libs = ["hello"]
                    self.cpp_info.build_modules["cmake"].append(module)
            """)
        target_alias = textwrap.dedent("""
            add_library(otherhello INTERFACE IMPORTED)
            target_link_libraries(otherhello INTERFACE hello::hello)
            """)
        client.save({"conanfile.py": conanfile, "target-alias.cmake": target_alias})
        client.run("create .")

        consumer = textwrap.dedent("""
            from conans import ConanFile, CMake

            class Conan(ConanFile):
                name = "consumer"
                version = "1.0"
                settings = "os", "compiler", "build_type", "arch"
                exports_sources = ["CMakeLists.txt"]
                generators = "cmake"
                requires = "hello/1.0"

                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
            """)
        cmakelists = textwrap.dedent("""
            cmake_minimum_required(VERSION 3.0)
            project(test)
            include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
            conan_basic_setup(TARGETS)
            get_target_property(tmp otherhello INTERFACE_LINK_LIBRARIES)
            message("otherhello link libraries: ${tmp}")
            """)
        client.save({"conanfile.py": consumer, "CMakeLists.txt": cmakelists})
        client.run("create .")
        assert "otherhello link libraries: hello::hello" in client.out



import os
import textwrap
import unittest

from conans.test.utils.tools import TestClient


class SettingsLoadTestCase(unittest.TestCase):
    def test_invalid_settings(self):
        client = TestClient()
        client.save({os.path.join(client.cache_folder, 'settings.yml'): """your buggy file"""})
        client.run("new -b hello/1.0")
        client.run("install .", assert_error=True)
        self.assertIn("ERROR: Invalid settings.yml format", client.out)

    def test_invalid_yaml(self):
        client = TestClient()
        client.save({os.path.join(client.cache_folder, 'settings.yml'):
                     textwrap.dedent("""
                        Almost:
                            - a
                            - valid
                          yaml
                     """)})
        client.run("new -b hello/1.0")
        client.run("install .", assert_error=True)
        self.assertIn("ERROR: Invalid settings.yml format: while parsing a block mapping",
                      client.out)



import textwrap

from conans.test.utils.tools import TestClient


def test_components_cycles():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class TestcycleConan(ConanFile):
            name = "testcycle"
            version = "1.0"

            def package_info(self):
                self.cpp_info.components["c"].requires = ["b"]
                self.cpp_info.components["b"].requires = ["a"]
                self.cpp_info.components["a"].requires = ["c"] # cycle!
        """)
    c.save({"conanfile.py": conanfile})
    c.run("create .", assert_error=True)
    assert "ERROR: There is a dependency loop in 'self.cpp_info.components' requires:" in c.out
    assert "a requires c"
    assert "b requires a"
    assert "c rquires b"



# coding=utf-8

import os
import textwrap
import unittest

import six
from jinja2 import Template

from conans.client.cache.cache import ClientCache
from conans.client.migrations_settings import settings_1_14_0
from conans.client.profile_loader import profile_from_args
from conans.errors import ConanException
from conans.test.utils.test_files import temp_folder
from conans.test.utils.mocks import TestBufferConanOutput
from conans.util.files import save


class SettingsCppStdTests(unittest.TestCase):

    def setUp(self):
        self.tmp_folder = temp_folder()
        self.cache = ClientCache(self.tmp_folder, TestBufferConanOutput())

    def _save_profile(self, cppstd=None, compiler_cppstd=None, filename="default"):
        fullpath = os.path.join(self.cache.profiles_path, filename)

        t = Template(textwrap.dedent("""
            [settings]
            os=Macos
            arch=x86_64
            compiler=apple-clang
            {% if compiler_cppstd %}compiler.cppstd={{ compiler_cppstd }}{% endif %}
            compiler.libcxx=libc++
            compiler.version=10.0
            {% if cppstd %}cppstd={{ cppstd }}{% endif %}
            """))

        save(fullpath, t.render(cppstd=cppstd, compiler_cppstd=compiler_cppstd))
        return filename

    def test_no_compiler_cppstd(self):
        # https://github.com/conan-io/conan/issues/5128
        fullpath = os.path.join(self.cache.profiles_path, "default")
        t = textwrap.dedent("""
            [settings]
            os=Macos
            arch=x86_64
            compiler=apple-clang
            compiler.libcxx=libc++
            compiler.version=10.0
            compiler.cppstd = 14
            """)
        save(self.cache.settings_path, settings_1_14_0)
        save(fullpath, t)
        r = profile_from_args(["default", ], [], [], [], [], cwd=self.tmp_folder, cache=self.cache)
        with six.assertRaisesRegex(self, ConanException,
                                   "'settings.compiler.cppstd' doesn't exist for 'apple-clang'"):
            r.process_settings(self.cache)

    def test_no_value(self):
        self._save_profile()

        r = profile_from_args(["default", ], [], [], [], [], cwd=self.tmp_folder, cache=self.cache)
        r.process_settings(self.cache)
        self.assertNotIn("compiler.cppstd", r.settings)
        self.assertNotIn("cppstd", r.settings)

    def test_value_none(self):
        self._save_profile(compiler_cppstd="None")

        r = profile_from_args(["default", ], [], [], [], [], cwd=self.tmp_folder, cache=self.cache)
        r.process_settings(self.cache)
        self.assertEqual(r.settings["compiler.cppstd"], "None")
        self.assertNotIn("cppstd", r.settings)

    def test_value_valid(self):
        self._save_profile(compiler_cppstd="11")

        r = profile_from_args(["default", ], [], [], [], [], cwd=self.tmp_folder, cache=self.cache)
        r.process_settings(self.cache)
        self.assertEqual(r.settings["compiler.cppstd"], "11")
        self.assertNotIn("cppstd", r.settings)

    def test_value_invalid(self):
        self._save_profile(compiler_cppstd="13")

        r = profile_from_args(["default", ], [], [], [], [], cwd=self.tmp_folder, cache=self.cache)
        with six.assertRaisesRegex(self, ConanException, "Invalid setting '13' is not a valid "
                                                         "'settings.compiler.cppstd' value"):
            r.process_settings(self.cache)
        self.assertNotIn("cppstd", r.settings)

    def test_value_duplicated_None(self):
        self._save_profile(compiler_cppstd="None", cppstd="None")

        r = profile_from_args(["default", ], [], [], [], [], cwd=self.tmp_folder, cache=self.cache)
        r.process_settings(self.cache)
        self.assertEqual(r.settings["compiler.cppstd"], "None")
        self.assertEqual(r.settings["cppstd"], "None")

    def test_value_duplicated(self):
        self._save_profile(compiler_cppstd="11", cppstd="11")

        r = profile_from_args(["default", ], [], [], [], [], cwd=self.tmp_folder, cache=self.cache)
        with six.assertRaisesRegex(self, ConanException, "Do not use settings 'compiler.cppstd'"
                                                         " together with 'cppstd'. Use only the"
                                                         " former one."):
            r.process_settings(self.cache)
        self.assertEqual(r.settings["compiler.cppstd"], "11")
        self.assertEqual(r.settings["cppstd"], "11")

    def test_value_different(self):
        self._save_profile(cppstd="14", compiler_cppstd="11")

        r = profile_from_args(["default", ], [], [], [], [], cwd=self.tmp_folder, cache=self.cache)
        with six.assertRaisesRegex(self, ConanException, "Do not use settings 'compiler.cppstd'"
                                                         " together with 'cppstd'. Use only the"
                                                         " former one"):
            r.process_settings(self.cache)

    def test_value_from_cppstd(self):
        self._save_profile(cppstd="11")

        r = profile_from_args(["default", ], [], [], [], [], cwd=self.tmp_folder, cache=self.cache)
        r.process_settings(self.cache)
        self.assertNotIn('compiler.cppstd', r.settings)
        self.assertEqual(r.settings["cppstd"], "11")



import os
import shutil


def copy_assets(src_folder, dst_folder, assets=None):
    assets_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "files")
    if src_folder:
        src_folder = os.path.join(assets_path, src_folder)
    assets = assets or os.listdir(src_folder)
    for asset in assets:
        s = os.path.join(src_folder, asset)
        d = os.path.join(dst_folder, asset)
        if os.path.isdir(s):
            shutil.copytree(s, d)
        else:
            shutil.copy2(s, d)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import platform
import unittest

import pytest

from conans.client import tools
from conans.client.conf import get_default_settings_yml
from conans.client.tools.env import environment_append
from conans.errors import ConanException
from conans.model.settings import Settings
from conans.test.utils.mocks import TestBufferConanOutput


@pytest.mark.tool_visual_studio
@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
class VCVarsArchTest(unittest.TestCase):
    output = TestBufferConanOutput()

    def assert_vcvars_command(self, settings, expected, output=None, **kwargs):
        output = output or self.output
        command = tools.vcvars_command(settings, output=output, **kwargs)
        command = command.replace('"', '').replace("'", "")
        self.assertTrue(command.endswith('vcvarsall.bat %s' % expected),
                        msg="'{}' doesn't end with 'vcvarsall.bat {}'".format(command, expected))

    def test_arch(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.compiler = 'Visual Studio'
        settings.compiler.version = '14'

        settings.arch = 'x86'
        self.assert_vcvars_command(settings, "x86")
        with environment_append({"PreferredToolArchitecture": "x64"}):
            self.assert_vcvars_command(settings, "amd64_x86")

        settings.arch = 'x86_64'
        self.assert_vcvars_command(settings, "amd64")

        settings.arch = 'armv7'
        self.assert_vcvars_command(settings, "amd64_arm")

        settings.arch = 'armv8'
        self.assert_vcvars_command(settings, "amd64_arm64")

        settings.arch = 'mips'
        with self.assertRaises(ConanException):
            tools.vcvars_command(settings, output=self.output)

        settings.arch_build = 'x86_64'
        settings.arch = 'x86'
        self.assert_vcvars_command(settings, "amd64_x86")

    def test_arch_override(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.compiler = 'Visual Studio'
        settings.compiler.version = '14'
        settings.arch = 'mips64'

        self.assert_vcvars_command(settings, "x86", arch='x86')
        self.assert_vcvars_command(settings, "amd64", arch='x86_64')
        self.assert_vcvars_command(settings, "amd64_arm", arch='armv7')
        self.assert_vcvars_command(settings, "amd64_arm64", arch='armv8')

        with self.assertRaises(ConanException):
            tools.vcvars_command(settings, arch='mips', output=self.output)

    def test_vcvars_ver_override(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.compiler = 'Visual Studio'
        settings.compiler.version = '15'
        settings.arch = 'x86_64'

        command = tools.vcvars_command(settings, vcvars_ver='14.14', output=self.output)
        self.assertIn('vcvarsall.bat', command)
        self.assertIn('-vcvars_ver=14.14', command)

        settings.compiler.version = '14'

        command = tools.vcvars_command(settings, vcvars_ver='14.14', output=self.output)
        self.assertIn('vcvarsall.bat', command)
        self.assertIn('-vcvars_ver=14.14', command)

    def test_winsdk_version_override(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.compiler = 'Visual Studio'
        settings.compiler.version = '15'
        settings.arch = 'x86_64'

        command = tools.vcvars_command(settings, winsdk_version='8.1', output=self.output)
        self.assertIn('vcvarsall.bat', command)
        self.assertIn('8.1', command)

        settings.compiler.version = '14'

        command = tools.vcvars_command(settings, winsdk_version='8.1', output=self.output)
        self.assertIn('vcvarsall.bat', command)
        self.assertIn('8.1', command)

    def test_windows_ce(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = 'WindowsCE'
        settings.compiler = 'Visual Studio'
        settings.compiler.version = '15'
        settings.arch = 'armv4i'
        self.assert_vcvars_command(settings, "x86")



import mock
from mock import Mock

from conan.tools.cmake import CMakeDeps
from conans import ConanFile, Settings
from conans.model.build_info import CppInfo
from conans.model.conanfile_interface import ConanFileInterface
from conans.model.dependencies import ConanFileDependencies, Requirement
from conans.model.env_info import EnvValues
from conans.model.ref import ConanFileReference


def test_cpp_info_name_cmakedeps():
    conanfile = ConanFile(Mock(), None)
    conanfile._conan_node = Mock()
    conanfile._conan_node.context = "host"
    conanfile.settings = "os", "compiler", "build_type", "arch"
    conanfile.initialize(Settings({"os": ["Windows"],
                                   "compiler": ["gcc"],
                                   "build_type": ["Release"],
                                   "arch": ["x86"]}), EnvValues())
    conanfile.settings.build_type = "Release"
    conanfile.settings.arch = "x86"

    cpp_info = CppInfo("mypkg", "dummy_root_folder1")
    cpp_info.set_property("cmake_target_name", "MySuperPkg1::MySuperPkg1")
    cpp_info.set_property("cmake_file_name", "ComplexFileName1")

    conanfile_dep = ConanFile(Mock(), None)
    conanfile_dep.cpp_info = cpp_info
    conanfile_dep._conan_node = Mock()
    conanfile_dep._conan_node.ref = ConanFileReference.loads("OriginalDepName/1.0")
    conanfile_dep._conan_node.context = "host"
    conanfile_dep.settings = conanfile.settings
    conanfile_dep.folders.set_base_package("/path/to/folder_dep")

    # FIXME: This will run infinite loop if conanfile.dependencies.host.topological_sort.
    #  Move to integration test
    with mock.patch('conans.ConanFile.dependencies', new_callable=mock.PropertyMock) as mock_deps:
        req = Requirement(ConanFileReference.loads("OriginalDepName/1.0"))
        mock_deps.return_value = ConanFileDependencies({req: ConanFileInterface(conanfile_dep)})

        cmakedeps = CMakeDeps(conanfile)
        files = cmakedeps.content
        assert "TARGET MySuperPkg1::MySuperPkg1" in files["ComplexFileName1-Target-release.cmake"]
        assert 'set(OriginalDepName_INCLUDE_DIRS_RELEASE ' \
               '"${OriginalDepName_PACKAGE_FOLDER_RELEASE}/include")' \
               in files["ComplexFileName1-release-x86-data.cmake"]


def test_cpp_info_name_cmakedeps_components():
    conanfile = ConanFile(Mock(), None)
    conanfile._conan_node = Mock()
    conanfile._conan_node.context = "host"
    conanfile.settings = "os", "compiler", "build_type", "arch"
    conanfile.initialize(Settings({"os": ["Windows"],
                                   "compiler": ["gcc"],
                                   "build_type": ["Release", "Debug"],
                                   "arch": ["x86", "x64"]}), EnvValues())
    conanfile.settings.build_type = "Debug"
    conanfile.settings.arch = "x64"

    cpp_info = CppInfo("mypkg", "dummy_root_folder1")
    cpp_info.set_property("cmake_target_name", "GlobalPkgName1::GlobalPkgName1")
    cpp_info.components["mycomp"].set_property("cmake_target_name", "GlobalPkgName1::MySuperPkg1")
    cpp_info.set_property("cmake_file_name", "ComplexFileName1")

    conanfile_dep = ConanFile(Mock(), None)
    conanfile_dep.cpp_info = cpp_info
    conanfile_dep.settings = conanfile.settings
    conanfile_dep._conan_node = Mock()
    conanfile_dep._conan_node.ref = ConanFileReference.loads("OriginalDepName/1.0")
    conanfile_dep._conan_node.context = "host"
    conanfile_dep.folders.set_base_package("/path/to/folder_dep")

    # FIXME: This will run infinite loop if conanfile.dependencies.host.topological_sort.
    #  Move to integration test
    with mock.patch('conans.ConanFile.dependencies', new_callable=mock.PropertyMock) as mock_deps:
        req = Requirement(ConanFileReference.loads("OriginalDepName/1.0"))
        mock_deps.return_value = ConanFileDependencies({req: ConanFileInterface(conanfile_dep)})

        cmakedeps = CMakeDeps(conanfile)
        files = cmakedeps.content
        assert "TARGET GlobalPkgName1::MySuperPkg1" in files["ComplexFileName1-Target-debug.cmake"]
        # Global variables for the packages
        # https://github.com/conan-io/conan/issues/11862
        assert 'set(OriginalDepName_INCLUDE_DIRS_DEBUG ' \
               '"${OriginalDepName_PACKAGE_FOLDER_DEBUG}/include")' \
               in files["ComplexFileName1-debug-x64-data.cmake"]
        # AND components
        assert 'set(OriginalDepName_GlobalPkgName1_MySuperPkg1_INCLUDE_DIRS_DEBUG ' \
               '"${OriginalDepName_PACKAGE_FOLDER_DEBUG}/include")' \
               in files["ComplexFileName1-debug-x64-data.cmake"]


def test_cmake_deps_links_flags():
    # https://github.com/conan-io/conan/issues/8703
    conanfile = ConanFile(Mock(), None)
    conanfile._conan_node = Mock()
    conanfile._conan_node.context = "host"
    conanfile.settings = "os", "compiler", "build_type", "arch"
    conanfile.initialize(Settings({"os": ["Windows"],
                                   "compiler": ["gcc"],
                                   "build_type": ["Release"],
                                   "arch": ["x86"]}), EnvValues())
    conanfile.settings.build_type = "Release"
    conanfile.settings.arch = "x86"

    cpp_info = CppInfo("mypkg", "dummy_root_folder1")
    # https://github.com/conan-io/conan/issues/8811 regression, fix with explicit - instead of /
    cpp_info.sharedlinkflags = ["-NODEFAULTLIB", "-OTHERFLAG"]
    cpp_info.exelinkflags = ["-OPT:NOICF"]
    cpp_info.objects = ["myobject.o"]
    conanfile_dep = ConanFile(Mock(), None)
    conanfile_dep.cpp_info = cpp_info
    conanfile_dep.settings = conanfile.settings
    conanfile_dep._conan_node = Mock()
    conanfile_dep._conan_node.ref = ConanFileReference.loads("mypkg/1.0")
    conanfile_dep._conan_node.context = "host"
    conanfile_dep.folders.set_base_package("/path/to/folder_dep")

    # FIXME: This will run infinite loop if conanfile.dependencies.host.topological_sort.
    #  Move to integration test
    with mock.patch('conans.ConanFile.dependencies', new_callable=mock.PropertyMock) as mock_deps:
        req = Requirement(ConanFileReference.loads("OriginalDepName/1.0"))
        mock_deps.return_value = ConanFileDependencies({req: ConanFileInterface(conanfile_dep)})

        cmakedeps = CMakeDeps(conanfile)
        files = cmakedeps.content
        data_cmake = files["mypkg-release-x86-data.cmake"]
        assert 'set(mypkg_SHARED_LINK_FLAGS_RELEASE "-NODEFAULTLIB;-OTHERFLAG")' in data_cmake
        assert 'set(mypkg_EXE_LINK_FLAGS_RELEASE "-OPT:NOICF")' in data_cmake
        assert 'set(mypkg_OBJECTS_RELEASE "${mypkg_PACKAGE_FOLDER_RELEASE}/myobject.o")' \
               in data_cmake


def test_component_name_same_package():
    """
    When the package and the component are the same the variables declared in data and linked
    to the target have to be the same.
    https://github.com/conan-io/conan/issues/9071"""
    conanfile = ConanFile(Mock(), None)
    conanfile._conan_node = Mock()
    conanfile._conan_node.context = "host"
    conanfile.settings = "os", "compiler", "build_type", "arch"
    conanfile.initialize(Settings({"os": ["Windows"],
                                   "compiler": ["gcc"],
                                   "build_type": ["Release"],
                                   "arch": ["x86"]}), EnvValues())
    conanfile.settings.build_type = "Release"
    conanfile.settings.arch = "x86"

    cpp_info = CppInfo("mypkg", "dummy_root_folder1")

    # We adjust the component with the same name as the package on purpose
    cpp_info.components["mypkg"].includedirs = ["includedirs1"]

    conanfile_dep = ConanFile(Mock(), None)
    conanfile_dep.cpp_info = cpp_info
    conanfile_dep.settings = conanfile.settings
    conanfile_dep._conan_node = Mock()
    conanfile_dep._conan_node.context = "host"
    conanfile_dep._conan_node.ref = ConanFileReference.loads("mypkg/1.0")
    conanfile_dep.folders.set_base_package("/path/to/folder_dep")

    # FIXME: This will run infinite loop if conanfile.dependencies.host.topological_sort.
    #  Move to integration test
    with mock.patch('conans.ConanFile.dependencies', new_callable=mock.PropertyMock) as mock_deps:
        req = Requirement(ConanFileReference.loads("mypkg/1.0"))
        mock_deps.return_value = ConanFileDependencies({req: ConanFileInterface(conanfile_dep)})

        cmakedeps = CMakeDeps(conanfile)
        files = cmakedeps.content
        target_cmake = files["mypkg-Target-release.cmake"]
        assert "$<$<CONFIG:Release>:${mypkg_mypkg_mypkg_INCLUDE_DIRS_RELEASE}> APPEND)" \
               in target_cmake

        data_cmake = files["mypkg-release-x86-data.cmake"]
        assert 'set(mypkg_mypkg_mypkg_INCLUDE_DIRS_RELEASE ' \
               '"${mypkg_PACKAGE_FOLDER_RELEASE}/includedirs1")' in data_cmake



import os
import platform
import textwrap
import unittest

import pytest

from conans.test.assets.genconanfile import GenConanfile
from conans.test.assets.sources import gen_function_cpp
from conans.test.utils.tools import TestClient

from conans.util.files import save, load


@pytest.mark.tool_cmake
@pytest.mark.skipif(platform.system() != "Windows", reason="Only for windows")
class CustomConfigurationTest(unittest.TestCase):
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMakeDeps
        class App(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            requires = "hello/0.1"

            def generate(self):
                cmake = CMakeDeps(self)
                if self.options["hello"].shared:
                    cmake.configuration = "ReleaseShared"
                cmake.generate()

            def imports(self):
                config = str(self.settings.build_type)
                if self.options["hello"].shared:
                    config = "ReleaseShared"
                self.copy("*.dll", src="bin", dst=config, keep_path=False)
        """)

    app = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])

    cmakelist = textwrap.dedent("""
        set(CMAKE_CONFIGURATION_TYPES Debug Release ReleaseShared CACHE STRING
            "Available build-types: Debug, Release and ReleaseShared")

        cmake_minimum_required(VERSION 2.8)
        project(App C CXX)

        set(CMAKE_PREFIX_PATH ${CMAKE_BINARY_DIR} ${CMAKE_PREFIX_PATH})
        set(CMAKE_MODULE_PATH ${CMAKE_BINARY_DIR} ${CMAKE_MODULE_PATH})

        set(CMAKE_CXX_FLAGS_RELEASESHARED ${CMAKE_CXX_FLAGS_RELEASE})
        set(CMAKE_C_FLAGS_RELEASESHARED ${CMAKE_C_FLAGS_RELEASE})
        set(CMAKE_EXE_LINKER_FLAGS_RELEASESHARED ${CMAKE_EXE_LINKER_FLAGS_RELEASE})

        find_package(hello REQUIRED)
        add_executable(app app.cpp)
        target_link_libraries(app PRIVATE hello::hello)
        """)

    def setUp(self):
        self.client = TestClient(path_with_spaces=False)
        self.client.run("new hello/0.1 -s")
        self.client.run("create . hello/0.1@ -s compiler.version=15 "
                        "-s build_type=Release -o hello:shared=True")
        self.client.run("create . hello/0.1@ -s compiler.version=15 "
                        "-s build_type=Release")

        # Prepare the actual consumer package
        self.client.save({"conanfile.py": self.conanfile,
                          "CMakeLists.txt": self.cmakelist,
                          "app.cpp": self.app})

    def test_generator_multi(self):
        settings = {"compiler": "Visual Studio",
                    "compiler.version": "15",
                    "arch": "x86_64",
                    "build_type": "Release",
                    }

        settings = " ".join('-s %s="%s"' % (k, v) for k, v in settings.items() if v)

        # Run the configure corresponding to this test case
        with self.client.chdir('build'):
            self.client.run("install .. %s -o hello:shared=True" % settings)
            self.client.run("install .. %s -o hello:shared=False" % settings)
            self.assertTrue(os.path.isfile(os.path.join(self.client.current_folder,
                                                        "hello-Target-releaseshared.cmake")))
            self.assertTrue(os.path.isfile(os.path.join(self.client.current_folder,
                                                        "hello-Target-release.cmake")))

            self.client.run_command('cmake .. -G "Visual Studio 15 Win64"')
            self.client.run_command('cmake --build . --config Release')
            self.client.run_command(r"Release\\app.exe")
            self.assertIn("hello/0.1: Hello World Release!", self.client.out)
            self.assertIn("main: Release!", self.client.out)
            self.client.run_command('cmake --build . --config ReleaseShared')
            self.client.run_command(r"ReleaseShared\\app.exe")
            self.assertIn("hello/0.1: Hello World Release!", self.client.out)
            self.assertIn("main: Release!", self.client.out)


@pytest.mark.tool_cmake
@pytest.mark.skipif(platform.system() != "Windows", reason="Only for windows")
class CustomSettingsTest(unittest.TestCase):
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMakeDeps

        class App(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            requires = "hello/0.1"

            def generate(self):
                cmake = CMakeDeps(self)
                #cmake.configurations.append("MyRelease") # NOT NECESSARY!!!
                cmake.generate()
        """)

    app = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])

    cmakelist = textwrap.dedent("""
        set(CMAKE_CONFIGURATION_TYPES Debug Release MyRelease CACHE STRING
            "Available build-types: Debug, Release and MyRelease")

        cmake_minimum_required(VERSION 2.8)
        project(App C CXX)

        set(CMAKE_PREFIX_PATH ${CMAKE_BINARY_DIR} ${CMAKE_PREFIX_PATH})
        set(CMAKE_MODULE_PATH ${CMAKE_BINARY_DIR} ${CMAKE_MODULE_PATH})

        set(CMAKE_CXX_FLAGS_MYRELEASE ${CMAKE_CXX_FLAGS_RELEASE})
        set(CMAKE_C_FLAGS_MYRELEASE ${CMAKE_C_FLAGS_RELEASE})
        set(CMAKE_EXE_LINKER_FLAGS_MYRELEASE ${CMAKE_EXE_LINKER_FLAGS_RELEASE})

        find_package(hello REQUIRED)
        add_executable(app app.cpp)
        target_link_libraries(app PRIVATE hello::hello)
        """)

    def setUp(self):
        self.client = TestClient(path_with_spaces=False)
        settings = load(self.client.cache.settings_path)
        settings = settings.replace("Release", "MyRelease")
        save(self.client.cache.settings_path, settings)
        self.client.run("new hello/0.1 -s")
        cmake = self.client.load("src/CMakeLists.txt")

        cmake = cmake.replace("cmake_minimum_required", """
            set(CMAKE_CONFIGURATION_TYPES Debug MyRelease Release CACHE STRING "Types")

            cmake_minimum_required""")
        cmake = cmake.replace("conan_basic_setup()", """
            conan_basic_setup()
            set(CMAKE_CXX_FLAGS_MYRELEASE ${CMAKE_CXX_FLAGS_RELEASE})
            set(CMAKE_C_FLAGS_MYRELEASE ${CMAKE_C_FLAGS_RELEASE})
            set(CMAKE_EXE_LINKER_FLAGS_MYRELEASE ${CMAKE_EXE_LINKER_FLAGS_RELEASE})
            """)
        self.client.save({"src/CMakeLists.txt": cmake})
        self.client.run("create . hello/0.1@ -s compiler.version=15 -s build_type=MyRelease")

        # Prepare the actual consumer package
        self.client.save({"conanfile.py": self.conanfile,
                          "CMakeLists.txt": self.cmakelist,
                          "app.cpp": self.app})

    def test_generator_multi(self):
        settings = {"compiler": "Visual Studio",
                    "compiler.version": "15",
                    "arch": "x86_64",
                    "build_type": "MyRelease",
                    }

        settings = " ".join('-s %s="%s"' % (k, v) for k, v in settings.items() if v)

        # Run the configure corresponding to this test case
        build_directory = os.path.join(self.client.current_folder, "build").replace("\\", "/")
        with self.client.chdir(build_directory):
            self.client.run("install .. %s" % settings)
            self.assertTrue(os.path.isfile(os.path.join(self.client.current_folder,
                                                        "hello-Target-myrelease.cmake")))

            self.client.run_command('cmake .. -G "Visual Studio 15 Win64"')
            self.client.run_command('cmake --build . --config MyRelease')
            self.client.run_command(r"MyRelease\\app.exe")
            self.assertIn("hello/0.1: Hello World Release!", self.client.out)
            self.assertIn("main: Release!", self.client.out)


@pytest.mark.tool_cmake
def test_changing_build_type():
    client = TestClient(path_with_spaces=False)
    dep_conanfile = textwrap.dedent(r"""
       from conans import ConanFile
       from conans.tools import save

       class Dep(ConanFile):
           settings = "build_type"
           def build(self):
               save("hello.h",
               '# include <iostream>\n'
               'void hello(){{std::cout<<"BUILD_TYPE={}!!";}}'.format(self.settings.build_type))
           def package(self):
               self.copy("*.h", dst="include")
           """)
    client.save({"conanfile.py": dep_conanfile})
    client.run("create . dep/0.1@ -s build_type=Release")
    client.run("create . dep/0.1@ -s build_type=Debug")

    cmakelists = textwrap.dedent("""
        set(CMAKE_CXX_COMPILER_WORKS 1)
        set(CMAKE_CXX_ABI_COMPILED 1)
        cmake_minimum_required(VERSION 3.15)
        project(App CXX)

        # NOTE: NO MAP necessary!!!
        find_package(dep REQUIRED)
        add_executable(app app.cpp)
        target_link_libraries(app PRIVATE dep::dep)
        """)
    app = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])
    pkg_conanfile = GenConanfile("pkg", "0.1").with_requires("dep/0.1").\
        with_generator("CMakeDeps").with_generator("CMakeToolchain").\
        with_settings("os", "compiler", "arch", "build_type")
    client.save({"conanfile.py": pkg_conanfile,
                 "CMakeLists.txt": cmakelists,
                 "app.cpp": app}, clean_first=True)

    # in MSVC multi-config -s pkg:build_type=Debug is not really necesary, toolchain do nothing
    # TODO: Challenge how to define consumer build_type for conanfile.txt
    client.run("install . -s pkg:build_type=Debug -s build_type=Release")
    client.run_command("cmake . -DCMAKE_TOOLCHAIN_FILE=conan_toolchain.cmake -DCMAKE_BUILD_TYPE=Debug")
    client.run_command("cmake --build . --config Debug")
    cmd = os.path.join(".", "Debug", "app") if platform.system() == "Windows" else "./app"
    client.run_command(cmd)
    assert "main: Debug!" in client.out
    assert "BUILD_TYPE=Release!!" in client.out




import os
import textwrap

from conans.test.utils.tools import TestClient


def test_layout_generate():
    """ Trying to leverage the layout knowledge at generate() time and put some files
    in the bindirs folders where executables will be built, like some dll from other
    dependencies.
    https://github.com/conan-io/conan/issues/12003
    """
    c = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save, load
        class Pkg(ConanFile):
            name = "pkg"
            version = "1.0"
            def layout(self):
                self.folders.build = "mybuild"
                self.folders.generators = "mybuild/generators"
                self.cpp.build.bindirs = ["bin"]
                self.cpp.build.libdirs = ["lib"]
                self.cpp.build.includedirs = ["include"]
            def generate(self):
                f = os.path.join(self.build_folder, self.cpp.build.bindir, "myfile.txt")
                save(self, f, "mybin!!!")
                f = os.path.join(self.build_folder, self.cpp.build.libdir, "myfile.txt")
                save(self, f, "mylib!!!")
                f = os.path.join(self.build_folder, self.cpp.build.includedir, "myfile.txt")
                save(self, f, "myinclude!!!")

            def build(self):
                self.output.info(load(self, os.path.join(self.cpp.build.libdir, "myfile.txt")))
                self.output.info(load(self, os.path.join(self.cpp.build.bindir, "myfile.txt")))
                self.output.info(load(self, os.path.join(self.cpp.build.includedir, "myfile.txt")))
        """)
    c.save({"conanfile.py": conanfile})
    c.run("create . ")
    assert "pkg/1.0: mybin!!!" in c.out
    assert "pkg/1.0: mylib!!!" in c.out
    assert "pkg/1.0: myinclude!!!" in c.out
    c.run("install .")
    assert os.path.exists(os.path.join(c.current_folder, "mybuild", "bin", "myfile.txt"))
    assert os.path.exists(os.path.join(c.current_folder, "mybuild", "lib", "myfile.txt"))
    assert os.path.exists(os.path.join(c.current_folder, "mybuild", "include", "myfile.txt"))





# coding=utf-8

import unittest

from conans.client.tools.scm import SCMBase
from conans.errors import ConanException


class RemoveCredentialsTest(unittest.TestCase):

    def test_http(self):
        expected_url = 'https://myrepo.com/path/to/repo.git'
        test_urls = ['https://myrepo.com/path/to/repo.git',
                     'https://username:password@myrepo.com/path/to/repo.git',
                     'https://username@myrepo.com/path/to/repo.git',
                     'https://gitlab-ci-token:1324@myrepo.com/path/to/repo.git',
                     ]

        for it in test_urls:
            self.assertEqual(expected_url, SCMBase._remove_credentials_url(it))

    def test_http_with_port_number(self):
        self.assertEqual('https://myrepo.com:8000/path/to/repo.git',
                         SCMBase._remove_credentials_url(
                             'https://username@myrepo.com:8000/path/to/repo.git'))

    def test_ssh(self):
        # Here, for ssh, we don't want to remove the user ('git' in this example)
        # URL-like syntax
        self.assertEqual('ssh://git@github.com:2222/conan-io/conan.git',
                         SCMBase._remove_credentials_url(
                             'ssh://git@github.com:2222/conan-io/conan.git'))
        # URL-like syntax with a password
        self.assertEqual('ssh://git@github.com:2222/conan-io/conan.git',
                         SCMBase._remove_credentials_url(
                             'ssh://git:password@github.com:2222/conan-io/conan.git'))
        self.assertEqual('ssh://github.com:2222/conan-io/conan.git',
                         SCMBase._remove_credentials_url(
                             'ssh://github.com:2222/conan-io/conan.git'))
        # scp-like syntax
        self.assertEqual('git@github.com:conan-io/conan.git',
                         SCMBase._remove_credentials_url(
                             'git@github.com:conan-io/conan.git'))

    def test_local_unix(self):
        self.assertEqual('file:///srv/git/project.git',
                         SCMBase._remove_credentials_url('file:///srv/git/project.git'))
        self.assertEqual('file:///srv/git/PROJECT.git',
                         SCMBase._remove_credentials_url('file:///srv/git/PROJECT.git'))

    def test_local_windows(self):
        self.assertEqual('file:///c:/srv/git/PROJECT',
                         SCMBase._remove_credentials_url('file:///c:/srv/git/PROJECT'))
        self.assertEqual('file:///C:/srv/git/PROJECT',
                         SCMBase._remove_credentials_url('file:///C:/srv/git/PROJECT'))

    def test_svn_ssh(self):
        self.assertEqual('svn+ssh://10.106.191.164/home/svn/shproject',
                         SCMBase._remove_credentials_url(
                             'svn+ssh://username:password@10.106.191.164/home/svn/shproject'))


class OutputMock(object):

    def __init__(self):
        self.out = list()

    def warn(self, text):
        self.out.append("WARN: " + text)


class GetUrlWithCredentialsTest(unittest.TestCase):

    def test_url(self):
        scm = SCMBase()
        self.assertEqual('http://github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("http://github.com/conan-io/conan.git"))

    def test_url_username(self):
        scm = SCMBase()
        self.assertEqual('http://user@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("http://user@github.com/conan-io/conan.git"))

    def test_url_password(self):
        scm = SCMBase()
        self.assertEqual('http://user:pass@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials(
                             "http://user:pass@github.com/conan-io/conan.git"))

    def test_url_with_user_param(self):
        scm = SCMBase(username="user")
        self.assertEqual('https://user@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("https://github.com/conan-io/conan.git"))

    def test_url_with_password_param(self):
        scm = SCMBase(password="pass")
        self.assertEqual('https://github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("https://github.com/conan-io/conan.git"))

    def test_url_with_user_password_param(self):
        scm = SCMBase(username="user", password="pass")
        self.assertEqual('https://user:pass@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("https://github.com/conan-io/conan.git"))

    def test_url_with_user_password_characters_param(self):
        scm = SCMBase(username="el niño", password="la contra%seña")
        self.assertEqual('https://el+ni%C3%B1o:la+contra%25se%C3%B1a@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("https://github.com/conan-io/conan.git"))

    def test_url_user_with_user_param(self):
        output = OutputMock()
        scm = SCMBase(username="user", output=output)
        self.assertEqual('https://dani@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("https://dani@github.com/conan-io/conan.git"))
        self.assertEqual(1, len(output.out))
        self.assertIn("WARN: SCM username got from URL, ignoring 'username' parameter", output.out)

    def test_url_user_with_password_param(self):
        scm = SCMBase(password="pass")
        self.assertEqual('https://dani:pass@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("https://dani@github.com/conan-io/conan.git"))

    def test_url_user_with_user_password_param(self):
        output = OutputMock()
        scm = SCMBase(username="user", password="pass", output=output)
        self.assertEqual('https://dani:pass@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("https://dani@github.com/conan-io/conan.git"))
        self.assertEqual(1, len(output.out))
        self.assertIn("WARN: SCM username got from URL, ignoring 'username' parameter", output.out)

    def test_url_user_pass_with_user_param(self):
        output = OutputMock()
        scm = SCMBase(username="user", output=output)
        self.assertEqual('http://dani:pass@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials(
                             "http://dani:pass@github.com/conan-io/conan.git"))
        self.assertEqual(1, len(output.out))
        self.assertIn("WARN: SCM username got from URL, ignoring 'username' parameter", output.out)

    def test_url_user_pass_with_password_param(self):
        output = OutputMock()
        scm = SCMBase(password="pass", output=output)
        self.assertEqual('http://dani:secret@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials(
                             "http://dani:secret@github.com/conan-io/conan.git"))
        self.assertEqual(1, len(output.out))
        self.assertIn("WARN: SCM password got from URL, ignoring 'password' parameter", output.out)

    def test_url_user_pass_with_user_password_param(self):
        output = OutputMock()
        scm = SCMBase(username="user", password="pass", output=output)
        self.assertEqual('http://dani:secret@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials(
                             "http://dani:secret@github.com/conan-io/conan.git"))
        self.assertEqual(2, len(output.out))
        self.assertIn("WARN: SCM username got from URL, ignoring 'username' parameter", output.out)
        self.assertIn("WARN: SCM password got from URL, ignoring 'password' parameter", output.out)

    def test_ssh(self):
        scm = SCMBase()
        self.assertEqual('ssh://github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("ssh://github.com/conan-io/conan.git"))

    def test_ssh_username_password(self):
        output = OutputMock()
        scm = SCMBase(username="dani", password="pass", output=output)
        self.assertEqual('ssh://dani@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("ssh://github.com/conan-io/conan.git"))
        self.assertEqual(1, len(output.out))
        self.assertIn("WARN: SCM password cannot be set for ssh url, ignoring parameter", output.out)

    def test_ssh_username(self):
        scm = SCMBase(username="dani")
        self.assertEqual('ssh://dani@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("ssh://github.com/conan-io/conan.git"))

    def test_ssh_password(self):
        output = OutputMock()
        scm = SCMBase(password="pass", output=output)
        self.assertEqual('ssh://github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("ssh://github.com/conan-io/conan.git"))
        self.assertEqual(1, len(output.out))
        self.assertIn("WARN: SCM password cannot be set for ssh url, ignoring parameter", output.out)

    def test_ssh_url_with_username_only_password(self):
        output = OutputMock()
        scm = SCMBase(password="pass", output=output)
        self.assertEqual('ssh://dani@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("ssh://dani@github.com/conan-io/conan.git"))
        self.assertEqual(1, len(output.out))
        self.assertIn("WARN: SCM password cannot be set for ssh url, ignoring parameter", output.out)

    def test_ssh_url_with_username_only_username(self):
        output = OutputMock()
        scm = SCMBase(username="dani", output=output)
        self.assertEqual('ssh://git@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("ssh://git@github.com/conan-io/conan.git"))
        self.assertIn("WARN: SCM username got from URL, ignoring 'username' parameter", output.out)

    def test_ssh_url_with_username_and_username_password(self):
        output = OutputMock()
        scm = SCMBase(password="pass", username="dani", output=output)
        self.assertEqual('ssh://git@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("ssh://git@github.com/conan-io/conan.git"))
        self.assertEqual(2, len(output.out))
        self.assertIn("WARN: SCM password cannot be set for ssh url, ignoring parameter", output.out)
        self.assertIn("WARN: SCM username got from URL, ignoring 'username' parameter", output.out)

    def test_ssh_url_with_username_password_and_only_password(self):
        output = OutputMock()
        scm = SCMBase(password="password", output=output)
        self.assertEqual('ssh://git@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("ssh://git:pass@github.com/conan-io/conan.git"))
        self.assertEqual(2, len(output.out))
        self.assertIn("WARN: SCM password cannot be set for ssh url, ignoring parameter", output.out)
        self.assertIn("WARN: Password in URL cannot be set for 'ssh' SCM type, removing it",
                      output.out)

    def test_ssh_url_with_username_password_and_only_username(self):
        output = OutputMock()
        scm = SCMBase(username="dani", output=output)
        self.assertEqual('ssh://git@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("ssh://git:pass@github.com/conan-io/conan.git"))
        self.assertEqual(2, len(output.out))
        self.assertIn("WARN: SCM username got from URL, ignoring 'username' parameter", output.out)
        self.assertIn("WARN: Password in URL cannot be set for 'ssh' SCM type, removing it",
                      output.out)

    def test_ssh_url_with_username_password_and_username_password(self):
        output = OutputMock()
        scm = SCMBase(password="password", username="dani", output=output)
        self.assertEqual("ssh://git@github.com/conan-io/conan.git",
                         scm.get_url_with_credentials("ssh://git:pass@github.com/conan-io/conan.git"))
        self.assertEqual(3, len(output.out))
        self.assertIn("WARN: SCM password cannot be set for ssh url, ignoring parameter", output.out)
        self.assertIn("WARN: SCM username got from URL, ignoring 'username' parameter", output.out)
        self.assertIn("WARN: Password in URL cannot be set for 'ssh' SCM type, removing it",
                      output.out)

    def test_scp(self):
        scm = SCMBase()
        self.assertEqual('git@github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("git@github.com/conan-io/conan.git"))

    def test_scp_only_password(self):
        output = OutputMock()
        scm = SCMBase(password="pass", output=output)
        self.assertEqual("git@github.com:conan-io/conan.git",
                         scm.get_url_with_credentials("git@github.com:conan-io/conan.git"))
        self.assertIn("WARN: SCM password cannot be set for scp url, ignoring parameter", output.out)

    def test_scp_only_username(self):
        output = OutputMock()
        scm = SCMBase(username="dani", output=output)
        self.assertEqual('git@github.com:conan-io/conan.git',
                         scm.get_url_with_credentials("git@github.com:conan-io/conan.git"))
        self.assertIn("WARN: SCM username got from URL, ignoring 'username' parameter", output.out)

    def test_scp_username_password(self):
        output = OutputMock()
        scm = SCMBase(password="pass", username="dani", output=output)
        self.assertEqual("git@github.com:conan-io/conan.git",
                         scm.get_url_with_credentials("git@github.com:conan-io/conan.git"))
        self.assertEqual(2, len(output.out))
        self.assertIn("WARN: SCM password cannot be set for scp url, ignoring parameter", output.out)
        self.assertIn("WARN: SCM username got from URL, ignoring 'username' parameter", output.out)

    def test_scp_url_username_password(self):
        output = OutputMock()
        scm = SCMBase(password="password", output=output)
        self.assertEqual('git:pass@github.com:conan-io/conan.git',
                         scm.get_url_with_credentials("git:pass@github.com:conan-io/conan.git"))
        self.assertIn("WARN: URL type not supported, ignoring 'username' and 'password' "
                      "parameters", output.out)

    def test_file_url(self):
        scm = SCMBase()
        self.assertEqual("file://path/to/.git", scm.get_url_with_credentials("file://path/to/.git"))

    def test_file_url_with_username_password_params(self):
        output = OutputMock()
        scm = SCMBase(username="user", password="pass", output=output)
        self.assertEqual('file://path/to/.git', scm.get_url_with_credentials("file://path/to/.git"))
        self.assertEqual(2, len(output.out))
        self.assertIn("WARN: SCM username cannot be set for file url, ignoring parameter",
                      output.out)
        self.assertIn("WARN: SCM password cannot be set for file url, ignoring parameter",
                      output.out)

    def test_git(self):
        scm = SCMBase()
        self.assertEqual('git://github.com/conan-io/conan.git',
                         scm.get_url_with_credentials("git://github.com/conan-io/conan.git"))

    def test_git_only_password(self):
        output = OutputMock()
        scm = SCMBase(password="pass", output=output)
        self.assertEqual("git://github.com/conan-io/conan.git",
                         scm.get_url_with_credentials("git://github.com/conan-io/conan.git"))
        self.assertIn("WARN: SCM password cannot be set for git url, ignoring parameter", output.out)

    def test_git_only_username(self):
        output = OutputMock()
        scm = SCMBase(username="dani", output=output)
        self.assertEqual("git://github.com/conan-io/conan.git",
                         scm.get_url_with_credentials("git://github.com/conan-io/conan.git"))
        self.assertIn("WARN: SCM username cannot be set for git url, ignoring parameter", output.out)

    def test_git_username_password(self):
        output = OutputMock()
        scm = SCMBase(password="pass", username="dani", output=output)
        self.assertEqual("git://github.com/conan-io/conan.git",
                         scm.get_url_with_credentials("git://github.com/conan-io/conan.git"))
        self.assertEqual(2, len(output.out))
        self.assertIn("WARN: SCM password cannot be set for git url, ignoring parameter", output.out)
        self.assertIn("WARN: SCM password cannot be set for git url, ignoring parameter", output.out)

    def test_git_url_username_password(self):
        output = OutputMock()
        scm = SCMBase(password="pass", output=output)
        self.assertEqual("git://github.com/conan-io/conan.git",
                         scm.get_url_with_credentials(
                             "git://user:pass@github.com/conan-io/conan.git"))
        self.assertEqual(2, len(output.out))
        self.assertIn("WARN: SCM password cannot be set for git url, ignoring parameter", output.out)
        self.assertIn("WARN: Username/Password in URL cannot be set for 'git' SCM type, removing it",
                      output.out)



import json
import unittest

from conans.model.graph_lock import LOCKFILE
from conans.test.utils.tools import TestClient, GenConanfile
from conans.util.env_reader import get_env


class GraphLockVersionRangeTest(unittest.TestCase):
    user_channel = "user/channel"
    consumer = GenConanfile("PkgB", "0.1").with_require("PkgA/[>=0.1]@user/channel")
    upload = False
    if get_env("TESTING_REVISIONS_ENABLED", False):
        ref_a = "PkgA/0.1@user/channel#fa090239f8ba41ad559f8e934494ee2a"
        pkg_id_a = "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9"
        prev_a = "0d561e10e25511b9bfa339d06360d7c1"
        ref_b = "PkgB/0.1@user/channel"
        rrev_b = "e8cabe5f1c737bcb8223b667f071842d"
        pkg_id_b = "5bf1ba84b5ec8663764a406f08a7f9ae5d3d5fb5"
        prev_b = "97d1695f4e456433cc5a1dfa14655a0f"
    else:
        ref_a = "PkgA/0.1@user/channel"
        pkg_id_a = "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9"
        prev_a = "0"
        ref_b = "PkgB/0.1@user/channel"
        rrev_b = "0"
        pkg_id_b = "5bf1ba84b5ec8663764a406f08a7f9ae5d3d5fb5"
        prev_b = "0"

    def setUp(self):
        client = TestClient(default_server_user=True)
        self.client = client
        client.save({"conanfile.py": GenConanfile("PkgA", "0.1")})
        client.run("create . %s" % self.user_channel)
        if self.upload:
            client.run("upload PkgA/0.1* --all --confirm")
            client.run("remove * -f")

        # Use a consumer with a version range
        client.save({"conanfile.py": self.consumer})
        if self.user_channel:
            user, channel = self.user_channel.split("/")
            client.run("lock create conanfile.py  --lockfile-out=conan.lock "
                       "--user=%s --channel=%s" % (user, channel))
        else:
            client.run("lock create conanfile.py --lockfile-out=conan.lock")

        self._check_lock()

        # If we create a new PkgA version
        client.save({"conanfile.py": GenConanfile("PkgA", "0.2")})
        client.run("create . %s" % self.user_channel)
        if self.upload:
            client.run("upload PkgA/0.2* --all --confirm")
            client.run("remove * -f")
        client.save({"conanfile.py": self.consumer})

    def _check_lock(self, rrev_b=None, prev_b=None, package_id_b=None):
        lock_file = self.client.load(LOCKFILE)
        lock_file_json = json.loads(lock_file)
        nodes = lock_file_json["graph_lock"]["nodes"]
        pkg_a = nodes["1"]

        self.assertEqual(pkg_a["ref"], self.ref_a)
        self.assertEqual(pkg_a["package_id"],  self.pkg_id_a)
        self.assertEqual(pkg_a["prev"], self.prev_a)

        pkg_b = nodes["0"]
        ref_b = self.ref_b if (rrev_b is None or rrev_b == "0") else "%s#%s" % (self.ref_b, rrev_b)
        self.assertEqual(pkg_b["ref"], ref_b)
        self.assertEqual(pkg_b.get("package_id"), package_id_b)
        self.assertEqual(pkg_b.get("prev"), prev_b)

    def test_install_lock(self):
        # Normal install will use it (use install-folder to not change graph-info)
        client = self.client
        client.run("install . -if=tmp")  # Output graph_info to temporary
        self.assertIn("PkgA/0.2", client.out)
        self.assertNotIn("PkgA/0.1", client.out)

        # Locked install will use PkgA/0.1
        # To use the stored graph_info.json, it has to be explicit in "--install-folder"
        client.run("install . -g=cmake --lockfile=conan.lock")
        self._check_lock()

        self.assertIn("PkgA/0.1", client.out)
        self.assertNotIn("PkgA/0.2", client.out)
        cmake = client.load("conanbuildinfo.cmake")
        self.assertIn("PkgA/0.1", cmake)
        self.assertNotIn("PkgA/0.2", cmake)

    def test_info_lock(self):
        client = self.client
        client.run("info . --lockfile=conan.lock")
        self.assertIn("PkgA/0.1", client.out)
        self.assertNotIn("PkgA/0.2", client.out)
        self._check_lock()

    def test_install_ref_lock(self):
        client = self.client
        # Make sure not using lockfile, will get PkgA/0.2
        client.run("install PkgA/[>=0.1]@%s -if=tmp" % self.user_channel)
        user_channel = "@%s" % self.user_channel if self.user_channel else ""
        if self.upload:
            self.assertIn("PkgA/0.2%s: Downloaded package" % user_channel, client.out)
        else:
            self.assertIn("PkgA/0.2%s: Already installed!" % user_channel, client.out)
        self.assertNotIn("PkgA/0.1", client.out)
        # Not using lockfile, even if one in the folder
        client.run("install PkgA/0.1@%s --install-folder=." % self.user_channel)
        if self.upload:
            self.assertIn("PkgA/0.1%s: Downloaded package" % user_channel, client.out)
        else:
            self.assertIn("PkgA/0.1%s: Already installed!" % user_channel, client.out)
        self.assertNotIn("PkgA/0.2", client.out)
        client.run("install PkgA/0.2@%s --install-folder=." % self.user_channel)
        self.assertIn("PkgA/0.2%s: Already installed!" % user_channel, client.out)
        self.assertNotIn("PkgA/0.1", client.out)
        self._check_lock()
        # Range locked one, outside of range, should raise
        client.run("install PkgA/[>=0.3]@%s --lockfile=conan.lock" % self.user_channel,
                   assert_error=True)
        self.assertIn("ERROR: Version ranges not allowed in 'PkgA/[>=0.3]", client.out)
        client.run("install PkgA/0.3@%s --lockfile=conan.lock" % self.user_channel,
                   assert_error=True)
        self.assertIn("Couldn't find 'PkgA/0.3", client.out)

        # inside range should be possible to find it
        client.run("install PkgA/0.1@%s --lockfile=conan.lock" % self.user_channel)
        self.assertIn("PkgA/0.1%s: Already installed!" % user_channel, client.out)
        self.assertNotIn("PkgA/0.2", client.out)
        self._check_lock()

    def test_export_lock(self):
        # locking a version range at export
        self.client.run("export . %s --lockfile=conan.lock --lockfile-out=conan.lock"
                        % self.user_channel)
        self._check_lock(self.rrev_b)

    def test_create_lock(self):
        # Create is also possible
        client = self.client
        client.run("create . PkgB/0.1@%s --lockfile=conan.lock --lockfile-out=conan.lock"
                   % self.user_channel)
        self.assertIn("PkgA/0.1", client.out)
        self.assertNotIn("PkgA/0.2", client.out)
        self._check_lock(self.rrev_b, self.prev_b, self.pkg_id_b)

    def test_create_test_lock(self):
        # Create with test_package is also possible
        client = self.client
        client.save({"test_package/conanfile.py": GenConanfile().with_test("pass")})
        client.run("create . PkgB/0.1@%s --lockfile=conan.lock  --lockfile-out=conan.lock"
                   % self.user_channel)
        self.assertIn("(test package)", client.out)
        self.assertIn("PkgA/0.1", client.out)
        self.assertNotIn("PkgA/0.2", client.out)
        self._check_lock(self.rrev_b, self.prev_b, self.pkg_id_b)

    def test_export_pkg(self):
        client = self.client
        client.run("export-pkg . PkgB/0.1@%s --lockfile=conan.lock --lockfile-out=conan.lock"
                   % self.user_channel)
        self._check_lock(self.rrev_b, self.prev_b, self.pkg_id_b)

        # Same, but modifying also PkgB Recipe
        client.save({"conanfile.py": str(self.consumer) + "\n#comment"})
        client.run("export-pkg . PkgB/0.1@%s --lockfile=conan.lock --force" % self.user_channel,
                   assert_error=True)
        self.assertIn("Attempt to modify locked PkgB/0.1", client.out)


class GraphLockVersionRangeUploadTest(GraphLockVersionRangeTest):
    upload = True


class GraphLockVersionRangeNoUserChannelTest(GraphLockVersionRangeTest):
    # This is exactly the same as above, but not using user/channel in packages
    # https://github.com/conan-io/conan/issues/5873
    user_channel = ""
    consumer = GenConanfile("PkgB", "0.1").with_require("PkgA/[>=0.1]")
    upload = False
    if get_env("TESTING_REVISIONS_ENABLED", False):
        ref_a = "PkgA/0.1#fa090239f8ba41ad559f8e934494ee2a"
        pkg_id_a = "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9"
        prev_a = "0d561e10e25511b9bfa339d06360d7c1"
        ref_b = "PkgB/0.1"
        rrev_b = "afa95143c0c11c46ad57670e1e0a0aa0"
        pkg_id_b = "5bf1ba84b5ec8663764a406f08a7f9ae5d3d5fb5"
        prev_b = "f97ac3d1bee62d55a35085dd42fa847a"
    else:
        ref_a = "PkgA/0.1"
        pkg_id_a = "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9"
        prev_a = "0"
        ref_b = "PkgB/0.1"
        rrev_b = "0"
        pkg_id_b = "5bf1ba84b5ec8663764a406f08a7f9ae5d3d5fb5"
        prev_b = "0"


class GraphLockVersionRangeNoUserChannelUploadTest(GraphLockVersionRangeNoUserChannelTest):
    upload = True


class GraphLockBuildRequireVersionRangeTest(GraphLockVersionRangeTest):
    user_channel = "user/channel"
    consumer = GenConanfile("PkgB", "0.1").with_build_requires("PkgA/[>=0.1]@user/channel")
    upload = False
    if get_env("TESTING_REVISIONS_ENABLED", False):
        ref_a = "PkgA/0.1@user/channel#fa090239f8ba41ad559f8e934494ee2a"
        pkg_id_a = "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9"
        prev_a = "0d561e10e25511b9bfa339d06360d7c1"
        ref_b = "PkgB/0.1@user/channel"
        rrev_b = "b6f49e5ba6dd3d64af09a2f288e71330"
        pkg_id_b = "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9"
        prev_b = "33a5634bbd9ec26b369d3900d91ea9a0"
    else:
        ref_a = "PkgA/0.1@user/channel"
        pkg_id_a = "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9"
        prev_a = "0"
        ref_b = "PkgB/0.1@user/channel"
        rrev_b = "0"
        pkg_id_b = "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9"
        prev_b = "0"


class GraphLockBuildRequireVersionRangeUploadTest(GraphLockBuildRequireVersionRangeTest):
    upload = True



import textwrap

import pytest

from conans.test.assets.cmake import gen_cmakelists
from conans.test.assets.genconanfile import GenConanfile
from conans.test.assets.sources import gen_function_cpp, gen_function_h
from conans.test.utils.tools import TestClient


@pytest.fixture(scope="module")
def client():
    t = TestClient()
    cpp = gen_function_cpp(name="mydep")
    h = gen_function_h(name="mydep")
    cmake = gen_cmakelists(libname="mydep", libsources=["mydep.cpp"])
    conanfile = textwrap.dedent("""
        import os
        from conans import ConanFile
        from conan.tools.cmake import CMake

        class Conan(ConanFile):
            name = "mydep"
            version = "1.0"
            settings = "os", "arch", "compiler", "build_type"
            exports_sources = "*.cpp", "*.h", "CMakeLists.txt"
            generators = "CMakeToolchain"

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()

            def package(self):
                self.copy("*.h", dst="include")
                self.copy("*.lib", dst="lib", keep_path=False)
                self.copy("*.dll", dst="bin", keep_path=False)
                self.copy("*.dylib*", dst="lib", keep_path=False)
                self.copy("*.so", dst="lib", keep_path=False)
                self.copy("*.a", dst="lib", keep_path=False)

            def package_info(self):

                self.cpp_info.set_property("cmake_find_mode", "both")

                self.cpp_info.set_property("cmake_file_name", "MyDep")
                self.cpp_info.set_property("cmake_target_name", "MyDepTarget::MyDepTarget")

                self.cpp_info.set_property("cmake_module_file_name", "mi_dependencia")
                self.cpp_info.set_property("cmake_module_target_name", "mi_dependencia_namespace::mi_dependencia_target")

                self.cpp_info.components["crispin"].libs = ["mydep"]
                self.cpp_info.components["crispin"].set_property("cmake_target_name",
                                                                 "MyDepTarget::MyCrispinTarget")
                self.cpp_info.components["crispin"].set_property("cmake_module_target_name",
                                                                 "mi_dependencia_namespace::mi_crispin_target")
        """)

    t.save({"conanfile.py": conanfile,
            "mydep.cpp": cpp,
            "mydep.h": h,
            "CMakeLists.txt": cmake})

    t.run("create .")
    return t


@pytest.mark.tool_cmake
def test_reuse_with_modules_and_config(client):
    cpp = gen_function_cpp(name="main")

    cmake_exe_config = """
    add_executable(myapp main.cpp)
    find_package(MyDep) # This one will find the config
    target_link_libraries(myapp MyDepTarget::MyCrispinTarget)
    """

    cmake_exe_module = """
    add_executable(myapp2 main.cpp)
    find_package(mi_dependencia) # This one will find the module
    target_link_libraries(myapp2 mi_dependencia_namespace::mi_crispin_target)
    """

    cmake = """
    set(CMAKE_CXX_COMPILER_WORKS 1)
    set(CMAKE_CXX_ABI_COMPILED 1)
    set(CMAKE_C_COMPILER_WORKS 1)
    set(CMAKE_C_ABI_COMPILED 1)

    cmake_minimum_required(VERSION 3.15)
    project(project CXX)
    {}
    """

    # test config
    conanfile = GenConanfile().with_name("myapp")\
        .with_cmake_build().with_exports_sources("*.cpp", "*.txt").with_require("mydep/1.0")
    client.save({"conanfile.py": conanfile,
                 "main.cpp": cpp,
                 "CMakeLists.txt": cmake.format(cmake_exe_config)})

    client.run("install . -if=install")
    client.run("build . -if=install")

    # test modules
    conanfile = GenConanfile().with_name("myapp")\
        .with_cmake_build().with_exports_sources("*.cpp", "*.txt").with_require("mydep/1.0")
    client.save({"conanfile.py": conanfile,
                 "main.cpp": cpp,
                 "CMakeLists.txt": cmake.format(cmake_exe_module)}, clean_first=True)

    client.run("install . -if=install")
    client.run("build . -if=install")


find_modes = [
    ("both", "both", ""),
    ("both", "both", "MODULE"),
    ("config", "config", ""),
    ("module", "module", ""),
    (None, "both", ""),
    (None, "both", "MODULE")
]


@pytest.mark.parametrize("find_mode_PKGA, find_mode_PKGB, find_mode_consumer", find_modes)
def test_transitive_modules_found(find_mode_PKGA, find_mode_PKGB, find_mode_consumer):
    """
    related to https://github.com/conan-io/conan/issues/10224
    modules files variables were set with the pkg_name_FOUND or pkg_name_VERSION
    instead of using filename_*, also there was missing doing a find_dependency of the
    requires packages to find_package transitive dependencies
    """
    client = TestClient()
    conan_pkg = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            {requires}
            def package_info(self):
                if "{mode}" != "None":
                    self.cpp_info.set_property("cmake_find_mode", "{mode}")
                self.cpp_info.set_property("cmake_file_name", "{filename}")
                self.cpp_info.set_property("cmake_module_file_name", "{module_filename}")
                self.cpp_info.defines.append("DEFINE_{filename}")
            """)

    consumer = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import CMake
        class Consumer(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            requires = "pkgb/1.0"
            generators = "CMakeDeps", "CMakeToolchain"
            exports_sources = "CMakeLists.txt"
            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
        """)

    cmakelist = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.1)
        project(test_package CXX)
        find_package(MYPKGB REQUIRED {find_mode})
        message("MYPKGB_VERSION: ${{MYPKGB_VERSION}}")
        message("MYPKGB_VERSION_STRING: ${{MYPKGB_VERSION_STRING}}")
        message("MYPKGB_INCLUDE_DIRS: ${{MYPKGB_INCLUDE_DIRS}}")
        message("MYPKGB_INCLUDE_DIR: ${{MYPKGB_INCLUDE_DIR}}")
        message("MYPKGB_LIBRARIES: ${{MYPKGB_LIBRARIES}}")

        get_target_property(linked_libs pkgb::pkgb INTERFACE_LINK_LIBRARIES)
        message("MYPKGB_LINKED_LIBRARIES: '${{linked_libs}}'")

        get_target_property(linked_deps pkgb_DEPS_TARGET INTERFACE_LINK_LIBRARIES)
        message("MYPKGB_DEPS_LIBRARIES: '${{linked_deps}}'")

        message("MYPKGB_DEFINITIONS: ${{MYPKGB_DEFINITIONS}}")
        """)

    client.save({"pkgb.py": conan_pkg.format(requires='requires="pkga/1.0"', filename='MYPKGB', module_filename='MYPKGB',
                                             mode=find_mode_PKGB),
                 "pkga.py": conan_pkg.format(requires='', filename='MYPKGA', module_filename='unicorns', mode=find_mode_PKGA),
                 "consumer.py": consumer,
                 "CMakeLists.txt": cmakelist.format(find_mode=find_mode_consumer)})
    client.run("create pkga.py pkga/1.0@")
    client.run("create pkgb.py pkgb/1.0@")
    client.run("create consumer.py consumer/1.0@")

    assert "MYPKGB_VERSION: 1.0" in client.out
    assert "MYPKGB_VERSION_STRING: 1.0" in client.out
    assert "MYPKGB_INCLUDE_DIRS:" in client.out
    assert "MYPKGB_INCLUDE_DIR:" in client.out
    # The MYPKG_LIBRARIES contains the target for the current package, but the target is linked
    # with the dependencies also:
    assert "MYPKGB_LIBRARIES: pkgb::pkgb" in client.out
    assert "MYPKGB_LINKED_LIBRARIES: '$<$<CONFIG:Release>:>;$<$<CONFIG:Release>:>;pkgb_DEPS_TARGET'" in client.out
    assert "MYPKGB_DEPS_LIBRARIES: '$<$<CONFIG:Release>:>;$<$<CONFIG:Release>:>;" \
           "$<$<CONFIG:Release>:pkga::pkga>'" in client.out

    assert "MYPKGB_DEFINITIONS: -DDEFINE_MYPKGB" in client.out
    assert "Conan: Target declared 'pkga::pkga'"

    if find_mode_PKGA == "module":
        assert 'Found unicorns: 1.0 (found version "1.0")' in client.out




import os
import platform
import textwrap
import unittest

import pytest

from conans.client import tools
from conans.model.ref import ConanFileReference, PackageReference
from conans.paths import CONANFILE
from conans.test.assets.autotools import gen_makefile_am, gen_configure_ac
from conans.test.assets.sources import gen_function_cpp, gen_function_h
from conans.test.utils.tools import NO_SETTINGS_PACKAGE_ID, TestClient


class AutoToolsConfigureTest(unittest.TestCase):

    @pytest.mark.skipif(platform.system() != "Linux", reason="Requires Autotools")
    @pytest.mark.tool_autotools()
    def test_autotools_real_install_dirs(self):
        body = gen_function_cpp(name="hello", msg="Hola Mundo!")
        header = gen_function_h(name="hello")
        main = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])
        makefile_am = gen_makefile_am(main="main", main_srcs="main.cpp", lib="libhello.a",
                                      lib_srcs="hello.cpp")
        configure_ac = gen_configure_ac()

        conanfile = textwrap.dedent("""
            from conans import ConanFile, AutoToolsBuildEnvironment

            class TestConan(ConanFile):
                name = "test"
                version = "1.0"
                settings = "os", "compiler", "arch", "build_type"
                exports_sources = "*"

                def build(self):
                    self.run("aclocal")
                    self.run("autoconf")
                    self.run("automake --add-missing --foreign")
                    autotools = AutoToolsBuildEnvironment(self)
                    autotools.configure()
                    autotools.make()
                    autotools.install()

                def package_id(self):
                    # easier to have same package_id for the test
                    self.info.header_only()
            """)
        client = TestClient()
        client.save({"conanfile.py": conanfile,
                     "configure.ac": configure_ac,
                     "Makefile.am": makefile_am,
                     "main.cpp": main,
                     "hello.h": header,
                     "hello.cpp": body})
        client.run("create . danimtb/testing")
        pref = PackageReference.loads("test/1.0@danimtb/testing:%s" % NO_SETTINGS_PACKAGE_ID)
        pkg_path = client.cache.package_layout(pref.ref).package(pref)

        [self.assertIn(folder, os.listdir(pkg_path)) for folder in ["lib", "bin"]]

        new_conanfile = conanfile.replace("autotools.configure()",
                                          "autotools.configure(args=['--bindir=${prefix}/superbindir', '--libdir=${prefix}/superlibdir'])")
        client.save({"conanfile.py": new_conanfile})
        client.run("create . danimtb/testing")
        [self.assertIn(folder, os.listdir(pkg_path)) for folder in ["superlibdir", "superbindir"]]
        [self.assertNotIn(folder, os.listdir(pkg_path)) for folder in ["lib", "bin"]]

    @pytest.mark.skipif(platform.system() == "Windows", reason="Not in Windows")
    def test_pkg_config_paths(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile, tools, AutoToolsBuildEnvironment

            class HelloConan(ConanFile):
                name = "Hello"
                version = "1.2.1"
                generators = %s

                def build(self):
                    tools.save("configure", "printenv")
                    self.run("chmod +x configure")
                    autot = AutoToolsBuildEnvironment(self)
                    autot.configure(%s)
            """)

        client.save({CONANFILE: conanfile % ("'txt'", "")})
        client.run("create . conan/testing")
        self.assertNotIn("PKG_CONFIG_PATH=", client.out)

        ref = ConanFileReference.loads("Hello/1.2.1@conan/testing")
        builds_folder = client.cache.package_layout(ref).builds()
        bf = os.path.join(builds_folder, os.listdir(builds_folder)[0])

        client.save({CONANFILE: conanfile % ("'pkg_config'", "")})
        client.run("create . conan/testing")
        self.assertIn("PKG_CONFIG_PATH=%s" % bf, client.out)

        # The previous values in the environment should be kept too
        with tools.environment_append({"PKG_CONFIG_PATH": "Some/value"}):
            client.run("create . conan/testing")
            self.assertIn("PKG_CONFIG_PATH=%s:Some/value" % bf, client.out)

        client.save({CONANFILE: conanfile % ("'pkg_config'",
                                             "pkg_config_paths=['/tmp/hello', 'foo']")})
        client.run("create . conan/testing")
        self.assertIn("PKG_CONFIG_PATH=/tmp/hello:%s/foo" % bf, client.out)

        # The previous values in the environment should be kept too
        with tools.environment_append({"PKG_CONFIG_PATH": "Some/value"}):
            client.run("create . conan/testing")
            self.assertIn("PKG_CONFIG_PATH=/tmp/hello:%s/foo:Some/value" % bf, client.out)



import textwrap

from conans.test.utils.tools import TestClient


def test_premakedeps():
    conanfile = textwrap.dedent("""
        [generators]
        PremakeDeps
        """)
    client = TestClient()
    client.save({"conanfile.txt": conanfile}, clean_first=True)
    client.run("install .")

    contents = client.load("conandeps.premake.lua")
    assert 'function conan_basic_setup()' in contents



import os
import textwrap

import pytest
from mock import patch

from conan.tools._compilers import architecture_flag, cppstd_flag
from conan.tools.intel import IntelCC
from conans.errors import ConanException
from conans.model.conf import ConfDefinition
from conans.test.utils.mocks import ConanFileMock, MockSettings


@pytest.mark.parametrize("os_,arch,expected", [
    ("Windows", "x86", "/Qm32"),
    ("Windows", "x86_64", "/Qm64"),
    ("Linux", "x86", "-m32"),
    ("Linux", "x86_64", "-m64")
])
def test_architecture_flag_if_intel_cc(os_, arch, expected):
    settings = MockSettings({
        "compiler": "intel-cc",
        "compiler.version": "2021.3",
        "compiler.mode": "classic",
        "arch": arch,
        "os": os_
    })
    flag = architecture_flag(settings)
    assert flag == expected


@pytest.mark.parametrize("cppstd,flag", [
    ("98", "c++98"),
    ("gnu98", "gnu++98"),
    ("03", "c++03"),
    ("gnu03", "gnu++03"),
    ("11", "c++11"),
    ("gnu11", "gnu++11"),
    ("14", "c++14"),
    ("gnu14", "gnu++14"),
    ("17", "c++17"),
    ("gnu17", "gnu++17"),
    ("20", "c++20"),
    ("gnu20", "gnu++20"),
    ("23", "c++2b"),
    ("gnu23", "gnu++2b"),
])
def test_cppstd_flag_if_intel_cc(cppstd, flag):
    settings = MockSettings({
        "compiler": "intel-cc",
        "compiler.version": "2021.3",
        "compiler.mode": "classic",
        "arch": "x86_64",
        "os": "Linux",
        "compiler.cppstd": cppstd
    })
    assert cppstd_flag(settings) == "-std=%s" % flag


@pytest.mark.parametrize("mode", ["icx", "dpcpp"])
def test_macos_not_supported_for_new_compilers(mode):
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({
        "compiler.version": "2021.3",
        "compiler.mode": mode,
        "os": "Darwin"
    })
    with pytest.raises(ConanException) as excinfo:
        IntelCC(conanfile)
    assert "macOS* is not supported for the icx/icpx or dpcpp compilers." in str(excinfo.value)


@pytest.mark.parametrize("os_", ["Windows", "Linux", "Darwin"])
def test_error_if_detected_intel_legacy_version(os_):
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({
        "compiler.version": "19.1",
        "compiler.mode": "classic",
        "os": os_
    })
    with pytest.raises(ConanException) as excinfo:
        IntelCC(conanfile)
    assert "You have to use 'intel' compiler which is meant for legacy" in str(excinfo.value)


@pytest.mark.parametrize("os_", ["Windows", "Linux", "Darwin"])
def test_classic_compiler_supports_every_os(os_):
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({
        "compiler.version": "2021.3",
        "compiler.mode": "classic",
        "os": os_,
        "arch": "x86_64"
    })
    assert IntelCC(conanfile).arch == "x86_64"


@pytest.mark.parametrize("mode,expected", [
    ("icx", "Intel C++ Compiler 2021"),
    ("dpcpp", "Intel(R) oneAPI DPC++ Compiler"),
    ("classic", "Intel C++ Compiler 19.2")
])
def test_check_ms_toolsets(mode, expected):
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({
        "compiler.version": "2021.3",
        "compiler.mode": mode,
        "os": "Windows"
    })
    assert IntelCC(conanfile).ms_toolset == expected


def test_installation_path_in_conf():
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({
        "compiler.version": "2021.3",
        "compiler.mode": "classic",
        "os": "Windows"
    })
    fake_path = "mysuper/path/to/intel/oneapi"
    conanfile.conf = ConfDefinition()
    conanfile.conf.loads(textwrap.dedent("""\
        tools.intel:installation_path=%s
    """ % fake_path))
    assert IntelCC(conanfile).installation_path == fake_path


@pytest.mark.parametrize("os_,call_command,setvars_file", [
    ("Windows", "call", "setvars.bat"),
    ("Linux", ".", "setvars.sh")
])
@patch("conan.tools.intel.intel_cc.platform.system")
def test_setvars_command_with_custom_arguments(platform_system, os_, call_command, setvars_file):
    platform_system.return_value = os_
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({
        "compiler.version": "2021.3",
        "compiler.mode": "icx",
        "os": os_
    })
    fake_path = "mysuper/path/to/intel/oneapi"
    args = "arg1 arg2 --force"
    conanfile.conf = ConfDefinition()
    conanfile.conf.loads(textwrap.dedent("""\
        tools.intel:installation_path=%s
        tools.intel:setvars_args=%s
    """ % (fake_path, args)))
    expected = '%s "%s" %s' % (call_command, os.path.join(fake_path, setvars_file), args)
    assert IntelCC(conanfile).command == expected



import os
import platform
import sys
import textwrap

import pytest

from conans.client.tools.apple import XCRun, to_apple_arch
from conans.test.assets.sources import gen_function_cpp, gen_function_h
from conans.test.utils.tools import TestClient

_conanfile_py = textwrap.dedent("""
from conan import ConanFile
from conan.tools.meson import Meson, MesonToolchain


class App(ConanFile):
    settings = "os", "arch", "compiler", "build_type"
    options = {"shared": [True, False], "fPIC": [True, False]}
    default_options = {"shared": False, "fPIC": True}

    def layout(self):
        self.folders.build = "build"

    def config_options(self):
        if self.settings.os == "Windows":
            del self.options.fPIC

    def generate(self):
        tc = MesonToolchain(self)
        tc.generate()

    def build(self):
        meson = Meson(self)
        meson.configure()
        meson.build()
""")

_meson_build = textwrap.dedent("""
project('tutorial', 'cpp')
add_global_arguments('-DSTRING_DEFINITION="' + get_option('STRING_DEFINITION') + '"',
                     language : 'cpp')
hello = library('hello', 'hello.cpp')
executable('demo', 'main.cpp', link_with: hello)
""")

_meson_options_txt = textwrap.dedent("""
option('STRING_DEFINITION', type : 'string', description : 'a string option')
""")


@pytest.mark.parametrize("arch, os_, os_version, sdk", [
    ('armv8', 'iOS', '10.0', 'iphoneos'),
    ('armv7', 'iOS', '10.0', 'iphoneos'),
    ('x86', 'iOS', '10.0', 'iphonesimulator'),
    ('x86_64', 'iOS', '10.0', 'iphonesimulator'),
    ('armv8', 'Macos', None, None)  # MacOS M1
])
@pytest.mark.tool_meson
@pytest.mark.skipif(sys.version_info.major == 2, reason="Meson not supported in Py2")
@pytest.mark.skipif(platform.system() != "Darwin", reason="requires Xcode")
def test_apple_meson_toolchain_cross_compiling(arch, os_, os_version, sdk):
    profile = textwrap.dedent("""
    include(default)

    [settings]
    os = {os}
    os.version = {os_version}
    os.sdk = {os_sdk}
    arch = {arch}
    compiler = apple-clang
    compiler.version = 12.0
    compiler.libcxx = libc++

    [conf]
    tools.apple:sdk_path={sdk_path}
    """)

    xcrun = XCRun(None, sdk)
    sdk_path = xcrun.sdk_path

    hello_h = gen_function_h(name="hello")
    hello_cpp = gen_function_cpp(name="hello", preprocessor=["STRING_DEFINITION"])
    app = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])
    profile = profile.format(
        os=os_,
        os_version=os_version,
        os_sdk=sdk,
        arch=arch,
        sdk_path=sdk_path)

    t = TestClient()
    t.save({"conanfile.py": _conanfile_py,
            "meson.build": _meson_build,
            "meson_options.txt": _meson_options_txt,
            "hello.h": hello_h,
            "hello.cpp": hello_cpp,
            "main.cpp": app,
            "profile_host": profile})

    t.run("install . --profile:build=default --profile:host=profile_host")
    t.run("build .")

    libhello = os.path.join(t.current_folder, "build", "libhello.a")
    assert os.path.isfile(libhello) is True
    demo = os.path.join(t.current_folder, "build", "demo")
    assert os.path.isfile(demo) is True

    lipo = xcrun.find('lipo')

    t.run_command('"%s" -info "%s"' % (lipo, libhello))
    assert "architecture: %s" % to_apple_arch(arch) in t.out

    t.run_command('"%s" -info "%s"' % (lipo, demo))
    assert "architecture: %s" % to_apple_arch(arch) in t.out

    # only check for iOS because one of the macos build variants is usually native
    if os_ == "iOS":
        content = t.load("conan_meson_cross.ini")
        assert "needs_exe_wrapper = true" in content


@pytest.mark.tool_meson
@pytest.mark.skipif(sys.version_info.major == 2, reason="Meson not supported in Py2")
# for Linux, build for x86 will require a multilib compiler
# for macOS, build for x86 is no longer supported by modern Xcode
@pytest.mark.skipif(platform.system() != "Windows", reason="requires Windows")
def test_windows_cross_compiling_x86():
    meson_build = textwrap.dedent("""
        project('tutorial', 'cpp')
        executable('demo', 'main.cpp')
        """)
    main_cpp = gen_function_cpp(name="main")
    profile_x86 = textwrap.dedent("""
        include(default)
        [settings]
        arch=x86
        """)

    client = TestClient()
    client.save({"conanfile.py": _conanfile_py,
                 "meson.build": meson_build,
                 "main.cpp": main_cpp,
                 "x86": profile_x86})
    profile_str = "--profile:build=default --profile:host=x86"
    client.run("install . %s" % profile_str)
    client.run("build .")
    client.run_command(os.path.join("build", "demo"))

    assert "main _M_IX86 defined" in client.out
    assert "main _MSC_VER19" in client.out
    assert "main _MSVC_LANG2014" in client.out


@pytest.mark.parametrize("arch, expected_arch", [('armv8', 'aarch64'),
                                                 ('armv7', 'arm'),
                                                 ('x86', 'i386'),
                                                 ('x86_64', 'x86_64')])
@pytest.mark.tool_meson
@pytest.mark.tool_android_ndk
@pytest.mark.skipif(platform.system() != "Darwin", reason="Android NDK only tested in MacOS for now")
@pytest.mark.skipif(sys.version_info.major == 2, reason="Meson not supported in Py2")
def test_android_meson_toolchain_cross_compiling(arch, expected_arch):
    profile_host = textwrap.dedent("""
    include(default)

    [settings]
    os = Android
    os.api_level = 21
    arch = {arch}

    [conf]
    tools.android:ndk_path={ndk_path}
    """)
    hello_h = gen_function_h(name="hello")
    hello_cpp = gen_function_cpp(name="hello", preprocessor=["STRING_DEFINITION"])
    app = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])
    profile_host = profile_host.format(
        arch=arch,
        ndk_path=os.getenv("TEST_CONAN_ANDROID_NDK")
    )

    client = TestClient()
    client.save({"conanfile.py": _conanfile_py,
                 "meson.build": _meson_build,
                 "meson_options.txt": _meson_options_txt,
                 "hello.h": hello_h,
                 "hello.cpp": hello_cpp,
                 "main.cpp": app,
                 "profile_host": profile_host})

    client.run("install . --profile:build=default --profile:host=profile_host")
    client.run("build .")
    content = client.load(os.path.join("conan_meson_cross.ini"))
    assert "needs_exe_wrapper = true" in content
    assert "Target machine cpu family: {}".format(expected_arch if expected_arch != "i386" else "x86") in client.out
    assert "Target machine cpu: {}".format(arch) in client.out
    libhello_name = "libhello.a" if platform.system() != "Windows" else "libhello.lib"
    libhello = os.path.join(client.current_folder, "build", libhello_name)
    demo = os.path.join(client.current_folder, "build", "demo")
    assert os.path.isfile(libhello)
    assert os.path.isfile(demo)

    # Check binaries architecture
    if platform.system() == "Darwin":
        client.run_command('objdump -f "%s"' % libhello)
        assert "architecture: %s" % expected_arch in client.out



import textwrap
import platform
import os

import pytest

from conans.test.utils.tools import TestClient


def _add_message_status_flags(client):
    cmakelists_path = os.path.join(client.current_folder, "CMakeLists.txt")
    with open(cmakelists_path, "a") as cmakelists_file:
        cmakelists_file.write('message(STATUS "CONAN_C_FLAGS: ${CONAN_C_FLAGS}")\n')
        cmakelists_file.write('message(STATUS "CONAN_CXX_FLAGS: ${CONAN_CXX_FLAGS}")\n')


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
@pytest.mark.parametrize("op_system,os_version,sdk,arch", [
    ("watchOS", "8.1", "watchos", "armv7k"),
    ("tvOS", "13.2", "appletvos", "armv8")
])
def test_cmake_apple_bitcode_arc_and_visibility_flags_enabled(op_system, os_version, sdk, arch):
    profile = textwrap.dedent("""
        include(default)
        [settings]
        os={}
        os.version={}
        os.sdk={}
        arch={}
        [conf]
        tools.apple:enable_bitcode=True
        tools.apple:enable_arc=True
        tools.apple:enable_visibility=True
    """.format(op_system, os_version, sdk, arch))

    client = TestClient(path_with_spaces=False)
    client.save({"host": profile}, clean_first=True)
    client.run("new hello/0.1 --template=cmake_lib")
    _add_message_status_flags(client)
    client.run("install . --profile:build=default --profile:host=host")
    toolchain = client.load(os.path.join("build", "generators", "conan_toolchain.cmake"))
    # bitcode
    assert 'set(CMAKE_XCODE_ATTRIBUTE_ENABLE_BITCODE "YES")' in toolchain
    assert 'set(CMAKE_XCODE_ATTRIBUTE_BITCODE_GENERATION_MODE "bitcode")' in toolchain
    assert 'set(BITCODE "-fembed-bitcode")' in toolchain
    # arc
    assert 'set(FOBJC_ARC "-fobjc-arc")' in toolchain
    assert 'set(CMAKE_XCODE_ATTRIBUTE_CLANG_ENABLE_OBJC_ARC "YES")' in toolchain
    # visibility
    assert 'set(CMAKE_XCODE_ATTRIBUTE_GCC_SYMBOLS_PRIVATE_EXTERN "NO")' in toolchain
    assert 'set(VISIBILITY "-fvisibility=default")' in toolchain

    client.run("create . --profile:build=default --profile:host=host -tf None")
    # flags
    assert "-- CONAN_C_FLAGS:  -fembed-bitcode -fobjc-arc" in client.out
    assert "-- CONAN_CXX_FLAGS:  -fembed-bitcode -fvisibility=default -fobjc-arc" in client.out
    assert "[100%] Built target hello" in client.out


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
@pytest.mark.parametrize("op_system,os_version,sdk,arch", [
    ("watchOS", "8.1", "watchos", "armv7k"),
    ("tvOS", "13.2", "appletvos", "armv8")
])
def test_cmake_apple_bitcode_arc_and_visibility_flags_enabled_and_xcode_generator(op_system, os_version, sdk, arch):
    """
    Testing when all the Bitcode, ARC and Visibility are enabled, and Xcode as generator.

    Note: When using CMake and Xcode as generator, the C/CXX flags do not need to be appended.
    """
    profile = textwrap.dedent("""
        include(default)
        [settings]
        os={}
        os.version={}
        os.sdk={}
        arch={}
        [conf]
        tools.apple:enable_bitcode=True
        tools.apple:enable_arc=True
        tools.apple:enable_visibility=True
    """.format(op_system, os_version, sdk, arch))

    client = TestClient(path_with_spaces=False)
    client.save({"host": profile}, clean_first=True)
    client.run("new hello/0.1 --template=cmake_lib")
    _add_message_status_flags(client)
    client.run("create . --profile:build=default --profile:host=host -tf None "
               "-c tools.cmake.cmaketoolchain:generator=Xcode")
    assert "** BUILD SUCCEEDED **" in client.out
    # flags are not appended when Xcode generator is used
    for line in str(client.out).splitlines():
        if "CONAN_C_FLAGS:" in line:
            assert "-- CONAN_C_FLAGS:" == line.strip()
        if "CONAN_CXX_FLAGS:" in line:
            assert "-- CONAN_CXX_FLAGS:  -stdlib=libc++" == line.strip()
            break


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
@pytest.mark.parametrize("op_system,os_version,sdk,arch", [
    ("watchOS", "8.1", "watchos", "armv7k"),
    ("tvOS", "13.2", "appletvos", "armv8")
])
def test_cmake_apple_bitcode_arc_and_visibility_flags_disabled(op_system, os_version, sdk, arch):
    profile = textwrap.dedent("""
        include(default)
        [settings]
        os={}
        os.version={}
        os.sdk={}
        arch={}
        [conf]
        tools.apple:enable_bitcode=False
        tools.apple:enable_arc=False
        tools.apple:enable_visibility=False
    """.format(op_system, os_version, sdk, arch))

    client = TestClient(path_with_spaces=False)
    client.save({"host": profile}, clean_first=True)
    client.run("new hello/0.1 --template=cmake_lib")
    _add_message_status_flags(client)
    client.run("install . --profile:build=default --profile:host=host")
    toolchain = client.load(os.path.join("build", "generators", "conan_toolchain.cmake"))
    # bitcode
    assert 'set(CMAKE_XCODE_ATTRIBUTE_ENABLE_BITCODE "NO")' in toolchain
    assert 'set(CMAKE_XCODE_ATTRIBUTE_BITCODE_GENERATION_MODE "bitcode")' not in toolchain
    assert 'set(BITCODE "-fembed-bitcode")' not in toolchain
    # arc
    assert 'set(FOBJC_ARC "-fno-objc-arc")' in toolchain
    assert 'set(CMAKE_XCODE_ATTRIBUTE_CLANG_ENABLE_OBJC_ARC "NO")' in toolchain
    # visibility
    assert 'set(CMAKE_XCODE_ATTRIBUTE_GCC_SYMBOLS_PRIVATE_EXTERN "YES")' in toolchain
    assert 'set(VISIBILITY "-fvisibility=hidden -fvisibility-inlines-hidden")' in toolchain

    client.run("create . --profile:build=default --profile:host=host -tf None")
    # flags
    assert "-- CONAN_C_FLAGS:   -fno-objc-arc" in client.out
    assert "-- CONAN_CXX_FLAGS:   -fvisibility=hidden -fvisibility-inlines-hidden -fno-objc-arc" in client.out
    assert "[100%] Built target hello" in client.out


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
@pytest.mark.parametrize("op_system,os_version,sdk,arch", [
    ("watchOS", "8.1", "watchos", "armv7k"),
    ("tvOS", "13.2", "appletvos", "armv8")
])
def test_cmake_apple_bitcode_arc_and_visibility_flags_are_none(op_system, os_version, sdk, arch):
    """
    Testing what happens when any of the Bitcode, ARC or Visibility configurations are not defined.
    """
    profile = textwrap.dedent("""
        include(default)
        [settings]
        os={}
        os.version={}
        os.sdk={}
        arch={}
    """.format(op_system, os_version, sdk, arch))

    client = TestClient(path_with_spaces=False)
    client.save({"host": profile}, clean_first=True)
    client.run("new hello/0.1 --template=cmake_lib")
    _add_message_status_flags(client)
    client.run("install . --profile:build=default --profile:host=host")
    toolchain = client.load(os.path.join("build", "generators", "conan_toolchain.cmake"))
    # bitcode
    assert 'set(CMAKE_XCODE_ATTRIBUTE_ENABLE_BITCODE "NO")' not in toolchain
    assert 'set(CMAKE_XCODE_ATTRIBUTE_BITCODE_GENERATION_MODE "bitcode")' not in toolchain
    assert 'set(BITCODE "-fembed-bitcode")' not in toolchain
    # arc
    assert 'set(FOBJC_ARC "-' not in toolchain
    assert 'set(CMAKE_XCODE_ATTRIBUTE_CLANG_ENABLE_OBJC_ARC' not in toolchain
    # visibility
    assert 'set(CMAKE_XCODE_ATTRIBUTE_GCC_SYMBOLS_PRIVATE_EXTERN' not in toolchain
    assert 'set(VISIBILITY "-' not in toolchain

    client.run("create . --profile:build=default --profile:host=host -tf None")
    # flags are not appended
    for flag in ["-fembed-bitcode", "-fno-objc-arc", "-fobjc-arc", "-fvisibility"]:
        assert flag not in client.out
    assert "[100%] Built target hello" in client.out



import six

from conans.client.graph.graph import RECIPE_CONSUMER, RECIPE_INCACHE
from conans.errors import ConanException
from conans.test.integration.graph.core.graph_manager_base import GraphManagerTest
from conans.test.utils.tools import GenConanfile


class TransitiveGraphTest(GraphManagerTest):
    def test_basic(self):
        # say/0.1
        deps_graph = self.build_graph(GenConanfile().with_name("Say").with_version("0.1"))
        self.assertEqual(1, len(deps_graph.nodes))
        node = deps_graph.root
        self.assertEqual(node.conanfile.name, "Say")
        self.assertEqual(len(node.dependencies), 0)
        self.assertEqual(len(node.dependants), 0)

    def test_transitive(self):
        # app -> libb0.1
        self.recipe_cache("libb/0.1")
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1"])

        deps_graph = self.build_consumer(consumer)

        self.assertEqual(2, len(deps_graph.nodes))
        app = deps_graph.root
        self.assertEqual(app.conanfile.name, "app")
        self.assertEqual(len(app.dependencies), 1)
        self.assertEqual(len(app.dependants), 0)
        self.assertEqual(app.recipe, RECIPE_CONSUMER)

        libb = app.dependencies[0].dst
        self.assertEqual(libb.conanfile.name, "libb")
        self.assertEqual(len(libb.dependencies), 0)
        self.assertEqual(len(libb.dependants), 1)
        self.assertEqual(libb.inverse_neighbors(), [app])
        self.assertEqual(list(libb.ancestors), [app])
        self.assertEqual(libb.recipe, RECIPE_INCACHE)

        self.assertEqual(list(app.public_closure), [libb])
        self.assertEqual(list(libb.public_closure), [])
        self.assertEqual(list(app.public_deps), [app, libb])
        self.assertEqual(list(libb.public_deps), list(app.public_deps))

    def test_transitive_two_levels(self):
        # app -> libb0.1 -> liba0.1
        self.recipe_cache("liba/0.1")
        self.recipe_cache("libb/0.1", ["liba/0.1"])
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1"])

        deps_graph = self.build_consumer(consumer)

        self.assertEqual(3, len(deps_graph.nodes))
        app = deps_graph.root
        libb = app.dependencies[0].dst
        liba = libb.dependencies[0].dst

        self._check_node(app, "app/0.1", deps=[libb], closure=[libb, liba])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app], closure=[liba])
        self._check_node(liba, "liba/0.1#123", dependents=[libb])

    def test_diamond(self):
        # app -> libb0.1 -> liba0.1
        #    \-> libc0.1 ->/
        self.recipe_cache("liba/0.1")
        self.recipe_cache("libb/0.1", ["liba/0.1"])
        self.recipe_cache("libc/0.1", ["liba/0.1"])
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "libc/0.1"])

        deps_graph = self.build_consumer(consumer)

        self.assertEqual(4, len(deps_graph.nodes))
        app = deps_graph.root
        libb = app.dependencies[0].dst
        libc = app.dependencies[1].dst
        liba = libb.dependencies[0].dst

        # No Revision??? Because of consumer?
        self._check_node(app, "app/0.1", deps=[libb, libc], closure=[libb, libc, liba])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app], closure=[liba])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app], closure=[liba])
        self._check_node(liba, "liba/0.1#123", dependents=[libb, libc])

    def test_consecutive_diamonds(self):
        # app -> libe0.1 -> libd0.1 -> libb0.1 -> liba0.1
        #    \-> libf0.1 ->/    \-> libc0.1 ->/
        self.recipe_cache("liba/0.1")
        self.recipe_cache("libb/0.1", ["liba/0.1"])
        self.recipe_cache("libc/0.1", ["liba/0.1"])
        self.recipe_cache("libd/0.1", ["libb/0.1", "libc/0.1"])
        self.recipe_cache("libe/0.1", ["libd/0.1"])
        self.recipe_cache("libf/0.1", ["libd/0.1"])
        consumer = self.recipe_consumer("app/0.1", ["libe/0.1", "libf/0.1"])

        deps_graph = self.build_consumer(consumer)

        self.assertEqual(7, len(deps_graph.nodes))
        app = deps_graph.root
        libe = app.dependencies[0].dst
        libf = app.dependencies[1].dst
        libd = libe.dependencies[0].dst
        libb = libd.dependencies[0].dst
        libc = libd.dependencies[1].dst
        liba = libc.dependencies[0].dst

        self._check_node(app, "app/0.1", deps=[libe, libf],
                         closure=[libe, libf, libd, libb, libc, liba])
        self._check_node(libe, "libe/0.1#123", deps=[libd], dependents=[app],
                         closure=[libd, libb, libc, liba])
        self._check_node(libf, "libf/0.1#123", deps=[libd], dependents=[app],
                         closure=[libd, libb, libc, liba])
        self._check_node(libd, "libd/0.1#123", deps=[libb, libc], dependents=[libe, libf],
                         closure=[libb, libc, liba])
        self._check_node(libc, "libc/0.1#123", deps=[liba], dependents=[libd], closure=[liba])
        self._check_node(libb, "libb/0.1#123", deps=[liba],  dependents=[libd], closure=[liba])
        self._check_node(liba, "liba/0.1#123", dependents=[libb, libc])

    def test_parallel_diamond(self):
        # app -> libb0.1 -> liba0.1
        #    \-> libc0.1 ->/
        #    \-> libd0.1 ->/
        self.recipe_cache("liba/0.1")
        self.recipe_cache("libb/0.1", ["liba/0.1"])
        self.recipe_cache("libc/0.1", ["liba/0.1"])
        self.recipe_cache("libd/0.1", ["liba/0.1"])

        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "libc/0.1", "libd/0.1"])

        deps_graph = self.build_consumer(consumer)

        self.assertEqual(5, len(deps_graph.nodes))
        app = deps_graph.root
        libb = app.dependencies[0].dst
        libc = app.dependencies[1].dst
        libd = app.dependencies[2].dst
        liba = libb.dependencies[0].dst

        self._check_node(app, "app/0.1", deps=[libb, libc, libd], closure=[libb, libc, libd, liba])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app], closure=[liba])
        self._check_node(libc, "libc/0.1#123", deps=[liba], dependents=[app], closure=[liba])
        self._check_node(libd, "libd/0.1#123", deps=[liba], dependents=[app], closure=[liba])
        self._check_node(liba, "liba/0.1#123", dependents=[libb, libc, libd])

    def test_nested_diamond(self):
        # app --------> libb0.1 -> liba0.1
        #    \--------> libc0.1 ->/
        #     \-> libd0.1 ->/
        self.recipe_cache("liba/0.1")
        self.recipe_cache("libb/0.1", ["liba/0.1"])
        self.recipe_cache("libc/0.1", ["liba/0.1"])
        self.recipe_cache("libd/0.1", ["libc/0.1"])

        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "libc/0.1", "libd/0.1"])

        deps_graph = self.build_consumer(consumer)

        self.assertEqual(5, len(deps_graph.nodes))
        app = deps_graph.root
        libb = app.dependencies[0].dst
        libc = app.dependencies[1].dst
        libd = app.dependencies[2].dst
        liba = libb.dependencies[0].dst

        self._check_node(app, "app/0.1", deps=[libb, libc, libd], closure=[libb, libd, libc, liba])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app], closure=[liba])
        self._check_node(libc, "libc/0.1#123", deps=[liba], dependents=[app, libd], closure=[liba])
        self._check_node(libd, "libd/0.1#123", deps=[libc], dependents=[app], closure=[libc, liba])
        self._check_node(liba, "liba/0.1#123", dependents=[libb, libc])

    def test_multiple_transitive(self):
        # https://github.com/conanio/conan/issues/4720
        # app -> libb0.1  -> libc0.1 -> libd0.1
        #    \--------------->/          /
        #     \------------------------>/
        self.recipe_cache("libd/0.1")
        self.recipe_cache("libc/0.1", ["libd/0.1"])
        self.recipe_cache("libb/0.1", ["libc/0.1"])
        consumer = self.recipe_consumer("app/0.1", ["libd/0.1", "libc/0.1", "libb/0.1"])

        deps_graph = self.build_consumer(consumer)

        self.assertEqual(4, len(deps_graph.nodes))
        app = deps_graph.root
        libd = app.dependencies[0].dst
        libc = app.dependencies[1].dst
        libb = app.dependencies[2].dst

        self._check_node(app, "app/0.1", deps=[libd, libc, libb], closure=[libb, libc, libd])
        self._check_node(libd, "libd/0.1#123", dependents=[app, libc])
        self._check_node(libb, "libb/0.1#123", deps=[libc], dependents=[app], closure=[libc, libd])
        self._check_node(libc, "libc/0.1#123", deps=[libd], dependents=[app, libb], closure=[libd])

    def test_diamond_conflict(self):
        # app -> libb0.1 -> liba0.1
        #    \-> libc0.1 -> liba0.2
        self.recipe_cache("liba/0.1")
        self.recipe_cache("liba/0.2")
        self.recipe_cache("libb/0.1", ["liba/0.1"])
        self.recipe_cache("libc/0.1", ["liba/0.2"])

        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "libc/0.1"])

        with six.assertRaisesRegex(self, ConanException, "Conflict in libc/0.1:\n"
            "    'libc/0.1' requires 'liba/0.2' while 'libb/0.1' requires 'liba/0.1'.\n"
            "    To fix this conflict you need to override the package 'liba' in your root "
            "package."):
            self.build_consumer(consumer)

    def test_loop(self):
        # app -> libc0.1 -> libb0.1 -> liba0.1 ->|
        #             \<-------------------------|
        self.recipe_cache("liba/0.1", ["libc/0.1"])
        self.recipe_cache("libb/0.1", ["liba/0.1"])
        self.recipe_cache("libc/0.1", ["libb/0.1"])

        consumer = self.recipe_consumer("app/0.1", ["libc/0.1"])

        with six.assertRaisesRegex(self, ConanException,
                                   "Loop detected in context host: 'liba/0.1' requires 'libc/0.1'"):
            self.build_consumer(consumer)

    def test_self_loop(self):
        self.recipe_cache("liba/0.1")
        consumer = self.recipe_consumer("liba/0.2", ["liba/0.1"])
        with six.assertRaisesRegex(self, ConanException,
                                   "Loop detected in context host: 'liba/0.2' requires 'liba/0.1'"):
            self.build_consumer(consumer)
